from __future__ import print_function, division

# Local import
from time_base import comparison_timer

isint_re = '''\
import re
int_regex = re.compile(r'[-+]?\d+$')
int_match = int_regex.match
nums = set([float, int])
def isint_re(x):
    """Function to simulate isint but with regular expressions."""
    t = type(x)
    return t == int if t in nums else bool(int_match(x))
'''

isint_try = '''\
def isint_try(x):
    """Function to simulate isint but with try/except."""
    try:
        int(x)
    except ValueError:
        return False
    else:
        return type(x) != float
'''

comparison_timer(['isint_re', isint_re],
                 ['isint_try', isint_try],
                 ['isint', 'from fastnumbers import isint'])



# -*- coding: utf-8 -*-
# Find the build location and add that to the path
from __future__ import print_function, division
import re
import sys
import os
import math
import unicodedata
from itertools import repeat
from platform import python_version_tuple
from pytest import raises
from hypothesis import given, assume, example
from hypothesis.strategies import (
    sampled_from,
    floats,
    integers,
    text,
    binary,
    lists,
)
import fastnumbers

if python_version_tuple()[0] == '3':
    long = int
    unichr = chr


# Predefine Unicode digits, numbers, and not those.
digits = []
numeric = []
not_numeric = []
for x in range(0x1FFFFF):
    try:
        a = unichr(x)
    except ValueError:
        break
    try:
        unicodedata.digit(a)
        digits.append(a)
    except ValueError:
        pass
    try:
        unicodedata.numeric(a)
        numeric.append(a)
    except ValueError:
        not_numeric.append(a)
not_numeric = sampled_from(not_numeric)
not_numeric = [not_numeric.example() for _ in range(1000)]  # This is too big otherwise
numeric_not_digit = [x for x in numeric if x not in digits]
numeric_not_digit_not_int = [x for x in numeric_not_digit if not unicodedata.numeric(x).is_integer()]


def a_number(s):
    s = s.strip()
    try:
        int(s)
    except ValueError:
        try:
            float(s)
        except ValueError:
            pass
        else:
            return True
    else:
        return True
    if python_version_tuple()[0] == '3':
        if isinstance(s, bytes):
            return False
    if re.match(r'\s*([-+]?\d+\.?\d*(?:[eE][-+]?\d+)?)\s*$', s, re.U):
        return True
    if re.match(r'\s*([-+]?\d+[lL]?)\s*$', s, re.U):
        return True
    if re.match(r'\s*([-+]?\.\d+(?:[eE][-+]?\d+)?)\s*$', s, re.U):
        return True
    if int(python_version_tuple()[0]) >= 3 or not isinstance(s, str):
        if s in numeric:
            return True
    return False


def baseN(num, b, numerals="0123456789abcdefghijklmnopqrstuvwxyz"):
    """
    Convert any integer to a Base-N string representation.
    Shamelessly stolen from http://stackoverflow.com/a/2267428/1399279
    """
    neg = num < 0
    num = abs(num)
    val = ((num == 0) and numerals[0]) or (baseN(num // b, b, numerals).lstrip(numerals[0]) + numerals[num % b])
    return '-' + val if neg else val


class DumbFloatClass(object):
    def __float__(self):
        raise ValueError("something here might go wrong")


class DumbIntClass(object):
    def __int__(self):
        raise ValueError("something here might go wrong")


def test_version():
    assert hasattr(fastnumbers, '__version__')


#################
# Sanity Checks #
#################


def test_fast_real_with_no_arguments_fails():
    with raises(TypeError):
        fastnumbers.fast_real(5, invalid='dummy')


def test_fast_float_with_no_arguments_fails():
    with raises(TypeError):
        fastnumbers.fast_float(5, invalid='dummy')


def test_fast_int_with_no_arguments_fails():
    with raises(TypeError):
        fastnumbers.fast_int(5, invalid='dummy')


def test_fast_forceint_with_no_arguments_fails():
    with raises(TypeError):
        fastnumbers.fast_forceint(5, invalid='dummy')


def test_isreal_with_no_arguments_fails():
    with raises(TypeError):
        fastnumbers.isreal(5, invalid='dummy')


def test_isfloat_with_no_arguments_fails():
    with raises(TypeError):
        fastnumbers.isfloat(5, invalid='dummy')


def test_isint_with_no_arguments_fails():
    with raises(TypeError):
        fastnumbers.isint(5, invalid='dummy')


def test_isintlike_with_no_arguments_fails():
    with raises(TypeError):
        fastnumbers.isintlike(5, invalid='dummy')


#############
# Fast Real #
#############


def test_fast_real_with_coerce_given_dumb_class_responds_to_internal_ValueError():
    x = DumbFloatClass()
    assert fastnumbers.fast_real(x, coerce=True) is x
    with raises(ValueError):
        fastnumbers.fast_real(x, coerce=True, raise_on_invalid=True)
    assert fastnumbers.fast_real(x, coerce=True, default=5) == 5


@given(floats())
def test_fast_real_given_float_returns_float(x):
    assume(not math.isnan(x))
    assert fastnumbers.fast_real(x, coerce=False) == x
    assert fastnumbers.fast_real(x, raise_on_invalid=True, coerce=False) == x
    assert fastnumbers.fast_real(x, None, True, coerce=False) == x
    assert isinstance(fastnumbers.fast_real(x, coerce=False), float)


@given(floats())
def test_fast_real_given_float_returns_int_if_intlike_with_coerce(x):
    assume(not math.isnan(x))
    assume(x.is_integer())
    assert fastnumbers.fast_real(x, coerce=True) == int(float(x))
    assert isinstance(fastnumbers.fast_real(x, coerce=True), (long, int))


@given(floats())
def test_fast_real_given_float_returns_float_or_int_with_coerce(x):
    assume(not math.isnan(x))
    assert fastnumbers.fast_real(x, coerce=True) == int(x) if x.is_integer() else x
    assert isinstance(fastnumbers.fast_real(x, coerce=True), (int, long) if x.is_integer() else float)


def test_fast_real_given_nan_returns_nan():
    assert math.isnan(fastnumbers.fast_real(float('nan')))


def test_fast_real_given_nan_returns_sub_value():
    assert fastnumbers.fast_real(float('nan'), nan=0) == 0


def test_fast_real_given_inf_returns_inf():
    assert math.isinf(fastnumbers.fast_real(float('inf')))


def test_fast_real_given_inf_returns_sub_value():
    assert fastnumbers.fast_real(float('inf'), inf=1000.0) == 1000.0


@given(floats())
@example(5.675088586167575e-116)
def test_fast_real_given_float_string_returns_float(x):
    assume(not math.isnan(x))
    y = repr(x)
    assert fastnumbers.fast_real(y, coerce=False) == x
    assert fastnumbers.fast_real(y, raise_on_invalid=True, coerce=False) == x
    assert fastnumbers.fast_real(y, None, True, coerce=False) == x
    assert isinstance(fastnumbers.fast_real(y, coerce=False), float)


@given(integers())
def test_fast_real_given_float_string_returns_int_with_coerce_with_intlike(x):
    y = repr(float(x))
    assert fastnumbers.fast_real(y, coerce=True) == int(float(x))
    assert isinstance(fastnumbers.fast_real(y, coerce=True), (long, int))


def test_fast_real_given_nan_string_returns_nan():
    assert math.isnan(fastnumbers.fast_real('nan'))
    assert math.isnan(fastnumbers.fast_real('NaN'))


def test_fast_real_with_nan_given_nan_string_returns_sub_value():
    assert fastnumbers.fast_real('nan', nan=0) == 0


def test_fast_real_given_inf_string_returns_inf():
    assert fastnumbers.fast_real('inf') == float('inf')
    assert fastnumbers.fast_real('-INF') == float('-inf')
    assert fastnumbers.fast_real('infinity') == float('inf')
    assert fastnumbers.fast_real('-infINIty') == float('-inf')


def test_fast_real_with_inf_given_inf_string_returns_sub_value():
    assert fastnumbers.fast_real('inf', inf=10000.0) == 10000.0


@given(floats(), integers(0, 100), integers(0, 100))
def test_fast_real_given_padded_float_strings_returns_float(x, y, z):
    assume(not math.isnan(x))
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert fastnumbers.fast_real(y, coerce=False) == x
    assert isinstance(fastnumbers.fast_real(y, coerce=False), float)


def test_fast_real_given_padded_nan_returns_nan():
    assert math.isnan(fastnumbers.fast_real('     nan '))


def test_fast_real_given_padded_inf_string_returns_inf():
    assert fastnumbers.fast_real('   inf        ') == float('inf')
    assert fastnumbers.fast_real('  -infINIty  ') == float('-inf')


@given(integers())
@example(1000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000)
def test_fast_real_given_int_returns_int(x):
    assert fastnumbers.fast_real(x) == x
    assert isinstance(fastnumbers.fast_real(x), (int, long))

@given(integers())
@example(1000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000)
def test_fast_real_given_int_returns_int_with_coerce(x):
    assert fastnumbers.fast_real(x, coerce=True) == x
    assert isinstance(fastnumbers.fast_real(x, coerce=True), (int, long))


@given(integers())
@example(40992764608243448035)
@example(-41538374848935286698640072416676709)
@example(240278958776173358420034462324117625982)
@example(1609422692302207451978552816956662956486)
@example(-121799354242674784350540853922878239740762834)
@example(32718704454132572934419741118153895444518280065843028297496525078)
@example(33684944745210074227862907273261282807602986571245071790093633147269)
def test_fast_real_given_int_string_returns_int(x):
    y = repr(x)
    assert fastnumbers.fast_real(y) == x
    assert isinstance(fastnumbers.fast_real(y), (int, long))


@given(integers(), integers(0, 100), integers(0, 100))
@example(40992764608243448035, 1, 1)
@example(-41538374848935286698640072416676709, 1, 1)
@example(240278958776173358420034462324117625982, 1, 1)
@example(1609422692302207451978552816956662956486, 1, 1)
@example(-121799354242674784350540853922878239740762834, 1, 1)
@example(32718704454132572934419741118153895444518280065843028297496525078, 1, 1)
@example(33684944745210074227862907273261282807602986571245071790093633147269, 1, 1)
def test_fast_real_given_padded_int_string_returns_int(x, y, z):
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert fastnumbers.fast_real(y) == x
    assert isinstance(fastnumbers.fast_real(y), (int, long))


@given(sampled_from(digits))
def test_fast_real_given_unicode_digit_returns_int(x):
    assert fastnumbers.fast_real(x) == unicodedata.digit(x)
    assert isinstance(fastnumbers.fast_real(x), (int, long))
    # Try padded as well
    assert fastnumbers.fast_real(u'   ' + x + u'   ') == unicodedata.digit(x)


@given(sampled_from(numeric_not_digit_not_int))
def test_fast_real_given_unicode_numeral_returns_float(x):
    assert fastnumbers.fast_real(x) == unicodedata.numeric(x)
    assert isinstance(fastnumbers.fast_real(x), float)
    # Try padded as well
    assert fastnumbers.fast_real(u'   ' + x + u'   ') == unicodedata.numeric(x)


@given(sampled_from(not_numeric))
def test_fast_real_given_unicode_non_numeral_returns_as_is(x):
    assert fastnumbers.fast_real(x) == x


@given(text(min_size=2))
def test_fast_real_given_unicode_of_more_than_one_char_returns_as_is(x):
    assume(not a_number(x))
    assert fastnumbers.fast_real(x) == x


@given(text() | binary())
@example('+')
@example('-')
@example('e8')
@example('.')
def test_fast_real_given_invalid_string_returns_string_as_is(x):
    assume(not a_number(x))
    assert fastnumbers.fast_real(x) is x


@given(text() | binary())
def test_fast_real_given_invalid_string_raises_ValueError_if_raise_on_invalid_is_True(x):
    assume(not a_number(x))
    with raises(ValueError):
        fastnumbers.fast_real(x, raise_on_invalid=True)


@given(lists(integers()))
def test_fast_real_given_invalid_type_raises_TypeError(x):
    with raises(TypeError):
        fastnumbers.fast_real(x)


@given(text() | binary())
def test_fast_real_returns_default_value_if_given_invalid_string(x):
    assume(not a_number(x))
    assert fastnumbers.fast_real(x, default=90) == 90
    assert fastnumbers.fast_real(x, 90) == 90


@given(text() | binary())
def test_fast_real_returns_raises_ValueError_if_raise_on_invalid_is_True_and_default_is_given(x):
    assume(not a_number(x))
    with raises(ValueError):
        fastnumbers.fast_real(x, default=90, raise_on_invalid=True)
        fastnumbers.fast_real(x, 90, True)


@given(integers() | floats())
def test_fast_real_returns_input_as_is_if_valid_and_key_is_given(x):
    fastnumbers.fast_real(x, key=len) == x
    fastnumbers.fast_real(str(x), key=len) == x


@given(text() | binary())
def test_fast_real_returns_transformed_input_if_invalid_and_key_is_given(x):
    assume(not a_number(x))
    fastnumbers.fast_real(x, key=len) == len(x)


##############
# Fast Float #
##############


def test_fast_float_given_dumb_class_responds_to_internal_ValueError():
    x = DumbFloatClass()
    assert fastnumbers.fast_float(x) is x
    with raises(ValueError):
        fastnumbers.fast_float(x, raise_on_invalid=True)
    assert fastnumbers.fast_float(x, default=5) == 5


@given(floats())
def test_fast_float_given_float_returns_float(x):
    assume(not math.isnan(x))
    assert fastnumbers.fast_float(x) == x
    assert fastnumbers.fast_float(x, None, True) == x
    assert fastnumbers.fast_float(x, raise_on_invalid=True) == x
    assert isinstance(fastnumbers.fast_float(x), float)


def test_fast_float_given_nan_returns_nan():
    assert math.isnan(fastnumbers.fast_float(float('nan')))


def test_fast_float_given_nan_returns_sub_value():
    assert fastnumbers.fast_float(float('nan'), nan=0) == 0


def test_fast_float_given_inf_returns_inf():
    assert math.isinf(fastnumbers.fast_float(float('inf')))


def test_fast_float_given_inf_returns_sub_value():
    assert fastnumbers.fast_float(float('inf'), inf=1000.0) == 1000.0


def test_fast_float_with_range_of_exponents_correctly_parses():
    for x in range(-300, 300):
        val = '1.0E{0:d}'.format(x)
        assert fastnumbers.fast_float(val) == float(val)
    for x in range(-300, 300):
        val = '1.0000000000E{0:d}'.format(x)
        assert fastnumbers.fast_float(val) == float(val)


@given(floats())
@example(5.675088586167575e-116)
def test_fast_float_given_float_string_returns_float(x):
    assume(not math.isnan(x))
    assume(not x.is_integer())
    y = repr(x)
    assert fastnumbers.fast_float(y) == x
    assert fastnumbers.fast_float(y, None, True) == x
    assert fastnumbers.fast_float(y, raise_on_invalid=True) == x
    assert isinstance(fastnumbers.fast_float(y), float)


def test_fast_float_given_nan_string_returns_nan():
    assert math.isnan(fastnumbers.fast_float('nan'))
    assert math.isnan(fastnumbers.fast_float('NaN'))


def test_fast_float_given_inf_string_returns_inf():
    assert fastnumbers.fast_float('inf') == float('inf')
    assert fastnumbers.fast_float('-INF') == float('-inf')
    assert fastnumbers.fast_float('infinity') == float('inf')
    assert fastnumbers.fast_float('-infINIty') == float('-inf')


@given(floats(), integers(0, 100), integers(0, 100))
def test_fast_float_given_padded_float_strings_returns_float(x, y, z):
    assume(not math.isnan(x))
    assume(not x.is_integer())
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert fastnumbers.fast_float(y) == x
    assert isinstance(fastnumbers.fast_float(y), float)


def test_fast_float_given_padded_nan_returns_nan():
    assert math.isnan(fastnumbers.fast_float('     nan '))


def test_fast_float_given_padded_inf_string_returns_inf():
    assert fastnumbers.fast_float('   inf        ') == float('inf')
    assert fastnumbers.fast_float('  -infINIty  ') == float('-inf')


@given(integers())
def test_fast_float_given_int_returns_float(x):
    assert fastnumbers.fast_float(x) == float(x)
    assert isinstance(fastnumbers.fast_float(x), float)


@given(integers())
@example(40992764608243448035)
@example(-41538374848935286698640072416676709)
@example(240278958776173358420034462324117625982)
@example(1609422692302207451978552816956662956486)
@example(-121799354242674784350540853922878239740762834)
@example(32718704454132572934419741118153895444518280065843028297496525078)
@example(33684944745210074227862907273261282807602986571245071790093633147269)
def test_fast_float_given_int_string_returns_float(x):
    y = repr(x)
    assert fastnumbers.fast_float(y) == float(x)
    assert isinstance(fastnumbers.fast_float(y), float)


@given(integers(), integers(0, 100), integers(0, 100))
@example(40992764608243448035, 1, 1)
@example(-41538374848935286698640072416676709, 1, 1)
@example(240278958776173358420034462324117625982, 1, 1)
@example(1609422692302207451978552816956662956486, 1, 1)
@example(-121799354242674784350540853922878239740762834, 1, 1)
@example(32718704454132572934419741118153895444518280065843028297496525078, 1, 1)
@example(33684944745210074227862907273261282807602986571245071790093633147269, 1, 1)
def test_fast_float_given_padded_int_string_returns_float(x, y, z):
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert fastnumbers.fast_float(y) == float(x)
    assert isinstance(fastnumbers.fast_float(y), float)


@given(sampled_from(digits))
def test_fast_float_given_unicode_digit_returns_float(x):
    assert fastnumbers.fast_float(x) == unicodedata.numeric(x)
    assert isinstance(fastnumbers.fast_float(x), float)
    # Try padded as well
    assert fastnumbers.fast_float(u'   ' + x + u'   ') == unicodedata.numeric(x)


@given(sampled_from(numeric))
def test_fast_float_given_unicode_numeral_returns_float(x):
    assert fastnumbers.fast_float(x) == unicodedata.numeric(x)
    assert isinstance(fastnumbers.fast_float(x), float)
    # Try padded as well
    assert fastnumbers.fast_float(u'   ' + x + u'   ') == unicodedata.numeric(x)


@given(sampled_from(not_numeric))
def test_fast_float_given_unicode_non_numeral_returns_as_is(x):
    assert fastnumbers.fast_float(x) == x


@given(text(min_size=2))
def test_fast_float_given_unicode_of_more_than_one_char_returns_as_is(x):
    assume(not a_number(x))
    assert fastnumbers.fast_float(x) == x


@given(text() | binary())
@example('+')
@example('-')
@example('e')
@example('e8')
@example('.')
def test_fast_float_given_invalid_string_returns_string_as_is(x):
    assume(not a_number(x))
    assert fastnumbers.fast_float(x) is x


@given(text() | binary())
def test_fast_float_given_invalid_string_raises_ValueError_if_raise_on_invalid_is_True(x):
    assume(not a_number(x))
    with raises(ValueError):
        fastnumbers.fast_float(x, None, True)
        fastnumbers.fast_float(x, raise_on_invalid=True)


@given(lists(integers()))
def test_fast_float_given_invalid_type_raises_TypeError(x):
    with raises(TypeError):
        fastnumbers.fast_float(x)


@given(text() | binary())
def test_fast_float_returns_default_value_if_given_invalid_string(x):
    assume(not a_number(x))
    assert fastnumbers.fast_float(x, default=90.0) == 90.0
    assert fastnumbers.fast_float(x, 90.0) == 90.0


@given(text() | binary())
def test_fast_float_returns_raises_ValueError_if_raise_on_invalid_is_True_and_default_is_given(x):
    assume(not a_number(x))
    with raises(ValueError):
        assert fastnumbers.fast_float(x, default=90.0, raise_on_invalid=True)
        assert fastnumbers.fast_float(x, 90.0, True)


@given(integers() | floats())
def test_fast_float_returns_input_as_is_if_valid_and_key_is_given(x):
    fastnumbers.fast_float(x, key=len) == x
    fastnumbers.fast_float(str(x), key=len) == x


@given(text() | binary())
def test_fast_float_returns_transformed_input_if_invalid_and_key_is_given(x):
    assume(not a_number(x))
    fastnumbers.fast_float(x, key=len) == len(x)


############
# Fast Int #
############


def test_fast_int_given_dumb_class_responds_to_internal_ValueError():
    x = DumbIntClass()
    assert fastnumbers.fast_int(x) is x
    with raises(ValueError):
        fastnumbers.fast_int(x, raise_on_invalid=True)
    assert fastnumbers.fast_int(x, default=5) == 5


def test_fast_int_given_invalid_base_errors_with_ValueError():
    with raises(ValueError):
        fastnumbers.fast_int('10', base=-1)
    with raises(ValueError):
        fastnumbers.fast_int('10', base=1)
    with raises(ValueError):
        fastnumbers.fast_int('10', base=37)


@given(floats())
def test_fast_int_given_float_returns_int(x):
    assume(not math.isnan(x))
    assume(not math.isinf(x))
    assert fastnumbers.fast_int(x) == int(x)
    assert fastnumbers.fast_int(x, raise_on_invalid=True) == int(x)
    assert fastnumbers.fast_int(x, None, True) == int(x)
    assert isinstance(fastnumbers.fast_int(x), (int, long))


def test_fast_int_given_nan_raises_ValueError_or_returns_as_is_or_returns_default():
    with raises(ValueError):
        fastnumbers.fast_int(float('nan'), raise_on_invalid=True)
    assert math.isnan(fastnumbers.fast_int(float('nan')))
    assert fastnumbers.fast_int(float('nan'), 'Sample') == 'Sample'


def test_fast_int_given_inf_raises_OverflowError():
    with raises(OverflowError):
        fastnumbers.fast_int(float('inf'), raise_on_invalid=True)
    assert math.isinf(fastnumbers.fast_int(float('inf')))
    assert fastnumbers.fast_int(float('inf'), 'Sample') == 'Sample'


@given(floats())
def test_fast_int_given_float_string_returns_string_as_is(x):
    assume(not math.isnan(x))
    assume(not x.is_integer())
    y = repr(x)
    assert fastnumbers.fast_int(y) is y


@given(floats())
def test_fast_int_given_float_intlike_string_returns_string_as_is(x):
    assume(not math.isnan(x))
    assume(x.is_integer())
    y = repr(x)
    assert fastnumbers.fast_int(y) is y


@given(floats())
@example(float('nan'))
@example(float('inf'))
def test_fast_int_given_float_string_raises_ValueError_if_raise_on_invalid_is_True(x):
    assume(not x.is_integer())
    y = repr(x)
    with raises(ValueError):
        fastnumbers.fast_int(y, None, True)
        fastnumbers.fast_int(y, raise_on_invalid=True)


@given(integers())
@example(1000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000)
def test_fast_int_given_int_returns_int(x):
    assert fastnumbers.fast_int(x) == x
    assert isinstance(fastnumbers.fast_int(x), (int, long))


@given(integers())
@example(40992764608243448035)
@example(-41538374848935286698640072416676709)
@example(240278958776173358420034462324117625982)
@example(1609422692302207451978552816956662956486)
@example(-121799354242674784350540853922878239740762834)
@example(32718704454132572934419741118153895444518280065843028297496525078)
@example(33684944745210074227862907273261282807602986571245071790093633147269)
def test_fast_int_given_int_string_returns_int(x):
    y = repr(x)
    assert fastnumbers.fast_int(y) == x
    assert isinstance(fastnumbers.fast_int(y), (int, long))
    for base in range(2, 36+1):
        if len(y) < 30:  # Avoid recursion error because of overly simple baseN function.
            assert fastnumbers.fast_int(baseN(x, base), base=base) == x
    assert fastnumbers.fast_int(bin(x), base=2) == x
    assert fastnumbers.fast_int(bin(x), base=0) == x
    assert fastnumbers.fast_int(oct(x), base=8) == x
    assert fastnumbers.fast_int(oct(x), base=0) == x
    assert fastnumbers.fast_int(hex(x), base=16) == x
    assert fastnumbers.fast_int(hex(x), base=0) == x


@given(integers(), integers(0, 100), integers(0, 100))
@example(40992764608243448035, 1, 1)
@example(-41538374848935286698640072416676709, 1, 1)
@example(240278958776173358420034462324117625982, 1, 1)
@example(1609422692302207451978552816956662956486, 1, 1)
@example(-121799354242674784350540853922878239740762834, 1, 1)
@example(32718704454132572934419741118153895444518280065843028297496525078, 1, 1)
@example(33684944745210074227862907273261282807602986571245071790093633147269, 1, 1)
def test_fast_int_given_padded_int_string_returns_int(x, y, z):
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert fastnumbers.fast_int(y) == x
    assert isinstance(fastnumbers.fast_int(y), (int, long))


@given(sampled_from(digits))
def test_fast_int_given_unicode_digit_returns_int(x):
    assert fastnumbers.fast_int(x) == unicodedata.digit(x)
    assert isinstance(fastnumbers.fast_int(x), (int, long))
    # Try padded as well
    assert fastnumbers.fast_int(u'   ' + x + u'   ') == unicodedata.digit(x)


@given(sampled_from(numeric_not_digit))
def test_fast_int_given_unicode_numeral_returns_as_is(x):
    assert fastnumbers.fast_int(x) == x


@given(sampled_from(not_numeric))
def test_fast_int_given_unicode_non_numeral_returns_as_is(x):
    assert fastnumbers.fast_int(x) == x


@given(text(min_size=2))
def test_fast_int_given_unicode_of_more_than_one_char_returns_as_is(x):
    assume(not a_number(x))
    assert fastnumbers.fast_int(x) == x


@given(text() | binary())
@example('+')
@example('-')
@example('e')
@example('e8')
@example('.')
def test_fast_int_given_invalid_string_returns_string_as_is(x):
    assume(not a_number(x))
    assert fastnumbers.fast_int(x) is x
    assert fastnumbers.fast_int(x, base=10) is x


@given(text() | binary())
def test_fast_int_given_invalid_string_raises_ValueError_if_raise_on_invalid_is_True(x):
    assume(not a_number(x))
    with raises(ValueError):
        fastnumbers.fast_int(x, raise_on_invalid=True)
        fastnumbers.fast_int(x, None, True)


@given(lists(integers()))
def test_fast_int_given_invalid_type_raises_TypeError(x):
    with raises(TypeError):
        fastnumbers.fast_int(x)


@given(text() | binary())
def test_fast_int_returns_default_value_if_given_invalid_string(x):
    assume(not a_number(x))
    assert fastnumbers.fast_int(x, default=90) == 90
    assert fastnumbers.fast_int(x, 90) == 90


@given(text() | binary())
def test_fast_int_returns_raises_ValueError_if_raise_on_invalid_is_True_and_default_is_given(x):
    assume(not a_number(x))
    with raises(ValueError):
        assert fastnumbers.fast_int(x, default=90, raise_on_invalid=True)
        assert fastnumbers.fast_int(x, 90, True)


@given(integers())
def test_fast_int_returns_input_as_is_if_valid_and_key_is_given(x):
    fastnumbers.fast_int(x, key=len) == x
    fastnumbers.fast_int(str(x), key=len) == x


@given(text() | binary())
def test_fast_int_returns_transformed_input_if_invalid_and_key_is_given(x):
    assume(not a_number(x))
    fastnumbers.fast_int(x, key=len) == len(x)


#################
# Fast Forceint #
#################


def test_fast_forceint_given_dumb_class_responds_to_internal_ValueError():
    x = DumbIntClass()
    assert fastnumbers.fast_forceint(x) is x
    with raises(ValueError):
        fastnumbers.fast_forceint(x, raise_on_invalid=True)
    assert fastnumbers.fast_forceint(x, default=5) == 5


@given(floats())
def test_fast_forceint_given_float_returns_int(x):
    assume(not math.isnan(x))
    assume(not math.isinf(x))
    assert fastnumbers.fast_forceint(x) == int(x)
    assert fastnumbers.fast_forceint(x, raise_on_invalid=True) == int(x)
    assert fastnumbers.fast_forceint(x, None, True) == int(x)
    assert isinstance(fastnumbers.fast_forceint(x), (int, long))


def test_fast_forceint_given_nan_raises_ValueError_or_returns_as_is_or_returns_default():
    with raises(ValueError):
        fastnumbers.fast_forceint(float('nan'), raise_on_invalid=True)
    assert math.isnan(fastnumbers.fast_forceint(float('nan')))
    assert fastnumbers.fast_forceint(float('nan'), 'Sample') == 'Sample'


def test_fast_forceint_given_inf_raises_OverflowError():
    with raises(OverflowError):
        fastnumbers.fast_forceint(float('inf'), raise_on_invalid=True)
    assert math.isinf(fastnumbers.fast_forceint(float('inf')))
    assert fastnumbers.fast_forceint(float('inf'), 'Sample') == 'Sample'


@given(floats())
def test_fast_forceint_given_float_string_returns_int(x):
    assume(not math.isnan(x))
    assume(not math.isinf(x))
    y = repr(x)
    assert fastnumbers.fast_forceint(y) == int(x)
    assert fastnumbers.fast_forceint(y, None, True) == int(x)
    assert fastnumbers.fast_forceint(y, raise_on_invalid=True) == int(x)
    assert isinstance(fastnumbers.fast_forceint(y), (int, long))


def test_fast_forceint_given_nan_string_raises_ValueError_with_raise_on_invalid_as_True():
    with raises(ValueError): 
        fastnumbers.fast_forceint('nan', raise_on_invalid=True)
        fastnumbers.fast_forceint('nan', None, True)


def test_fast_forceint_given_inf_string_raises_OverflowError_with_raise_on_invalid_as_True():
    with raises(OverflowError):
        fastnumbers.fast_forceint('inf', raise_on_invalid=True)
        fastnumbers.fast_forceint('-infinity', raise_on_invalid=True)
        fastnumbers.fast_forceint('inf', None, True)
        fastnumbers.fast_forceint('-infinity', None, True)


@given(floats(), integers(0, 100), integers(0, 100))
def test_fast_forceint_given_padded_float_strings_returns_int(x, y, z):
    assume(not math.isnan(x))
    assume(not math.isinf(x))
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert fastnumbers.fast_forceint(y) == int(x)
    assert isinstance(fastnumbers.fast_forceint(y), (int, long))


@given(integers())
@example(1000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000)
def test_fast_forceint_given_int_returns_int(x):
    assert fastnumbers.fast_forceint(x) == x
    assert isinstance(fastnumbers.fast_forceint(x), (int, long))


@given(integers())
@example(40992764608243448035)
@example(-41538374848935286698640072416676709)
@example(240278958776173358420034462324117625982)
@example(1609422692302207451978552816956662956486)
@example(-121799354242674784350540853922878239740762834)
@example(32718704454132572934419741118153895444518280065843028297496525078)
@example(33684944745210074227862907273261282807602986571245071790093633147269)
def test_fast_forceint_given_int_string_returns_int(x):
    y = repr(x)
    assert fastnumbers.fast_forceint(y) == x
    assert isinstance(fastnumbers.fast_forceint(y), (int, long))


@given(integers(), integers(0, 100), integers(0, 100))
@example(40992764608243448035, 1, 1)
@example(-41538374848935286698640072416676709, 1, 1)
@example(240278958776173358420034462324117625982, 1, 1)
@example(1609422692302207451978552816956662956486, 1, 1)
@example(-121799354242674784350540853922878239740762834, 1, 1)
@example(32718704454132572934419741118153895444518280065843028297496525078, 1, 1)
@example(33684944745210074227862907273261282807602986571245071790093633147269, 1, 1)
def test_fast_forceint_given_padded_int_string_returns_int(x, y, z):
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert fastnumbers.fast_forceint(y) == x
    assert isinstance(fastnumbers.fast_forceint(y), (int, long))


@given(sampled_from(digits))
def test_fast_forceint_given_unicode_digit_returns_int(x):
    assert fastnumbers.fast_forceint(x) == unicodedata.digit(x)
    assert isinstance(fastnumbers.fast_forceint(x), (int, long))
    # Try padded as well
    assert fastnumbers.fast_forceint(u'   ' + x + u'   ') == unicodedata.digit(x)


@given(sampled_from(numeric))
def test_fast_forceint_given_unicode_numeral_returns_int(x):
    assert fastnumbers.fast_forceint(x) == int(unicodedata.numeric(x))
    assert isinstance(fastnumbers.fast_forceint(x), (int, long))
    # Try padded as well
    assert fastnumbers.fast_forceint(u'   ' + x + u'   ') == int(unicodedata.numeric(x))


@given(sampled_from(not_numeric))
def test_fast_forceint_given_unicode_non_numeral_returns_as_is(x):
    assert fastnumbers.fast_forceint(x) == x


@given(text(min_size=2))
def test_fast_forceint_given_unicode_of_more_than_one_char_returns_as_is(x):
    assume(not a_number(x))
    assert fastnumbers.fast_int(x) == x


@given(text() | binary())
@example('+')
@example('-')
@example('e')
@example('e8')
@example('.')
def test_fast_forceint_given_invalid_string_returns_string_as_is(x):
    assume(not a_number(x))
    assert fastnumbers.fast_forceint(x) is x


@given(text() | binary())
def test_fast_forceint_given_invalid_string_raises_ValueError_if_raise_on_invalid_is_True(x):
    assume(not a_number(x))
    with raises(ValueError):
        fastnumbers.fast_forceint(x, raise_on_invalid=True)
        fastnumbers.fast_forceint(x, None, True)


@given(lists(integers()))
def test_fast_forceint_given_invalid_type_raises_TypeError(x):
    with raises(TypeError):
        fastnumbers.fast_forceint(x)


@given(text() | binary())
def test_fast_forceint_returns_default_value_if_given_invalid_string(x):
    assume(not a_number(x))
    assert fastnumbers.fast_forceint(x, default=90.0) == 90.0
    assert fastnumbers.fast_forceint(x, 90.0) == 90.0


@given(text() | binary())
def test_fast_forceint_returns_raises_ValueError_if_raise_on_invalid_is_True_and_default_is_given(x):
    assume(not a_number(x))
    with raises(ValueError):
        assert fastnumbers.fast_forceint(x, default=90.0, raise_on_invalid=True)
        assert fastnumbers.fast_forceint(x, 90.0, True)


@given(integers() | floats())
def test_fast_forceint_returns_input_as_is_if_valid_and_key_is_given(x):
    assume(not math.isnan(x))
    assume(not math.isinf(x))
    fastnumbers.fast_forceint(x, key=len) == int(x)
    fastnumbers.fast_forceint(str(x), key=len) == int(x)


@given(text() | binary())
def test_fast_forceint_returns_transformed_input_if_invalid_and_key_is_given(x):
    assume(not a_number(x))
    fastnumbers.fast_forceint(x, key=len) == len(x)


###########
# Is Real #
###########


@given(integers())
def test_isreal_returns_True_if_given_int(x):
    assert fastnumbers.isreal(x)
    assert fastnumbers.isreal(x, num_only=True)


@given(floats())
def test_isreal_returns_True_if_given_float(x):
    assert fastnumbers.isreal(x)
    assert fastnumbers.isreal(x, num_only=True)


@given(integers())
def test_isreal_returns_False_if_given_int_and_str_only_is_True(x):
    assert not fastnumbers.isreal(x, str_only=True)


@given(floats())
def test_isreal_returns_False_if_given_float_and_str_only_is_True(x):
    assert not fastnumbers.isreal(x, str_only=True)


@given(integers(), integers(0, 100), integers(0, 100))
def test_isreal_returns_True_if_given_int_string_padded_or_not(x, y, z):
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert fastnumbers.isreal(repr(x))
    assert fastnumbers.isreal(repr(x), str_only=True)
    assert fastnumbers.isreal(y)


@given(floats(), integers(0, 100), integers(0, 100))
def test_isreal_returns_True_if_given_float_string_padded_or_not(x, y, z):
    assume(not math.isnan(x))
    assume(not math.isinf(x))
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert fastnumbers.isreal(repr(x))
    assert fastnumbers.isreal(repr(x), str_only=True)
    assert fastnumbers.isreal(y)


@given(integers())
def test_isreal_returns_False_if_given_string_and_num_only_is_True(x):
    assert not fastnumbers.isreal(repr(x), num_only=True)


@given(sampled_from(digits))
def test_isreal_given_unicode_digit_returns_True(x):
    assert fastnumbers.isreal(x)
    # Try padded as well
    assert fastnumbers.isreal(u'   ' + x + u'   ')


@given(sampled_from(numeric_not_digit_not_int))
def test_isreal_given_unicode_numeral_returns_True(x):
    assert fastnumbers.isreal(x)
    # Try padded as well
    assert fastnumbers.isreal(u'   ' + x + u'   ')


@given(sampled_from(not_numeric))
def test_isreal_given_unicode_non_numeral_returns_False(x):
    assert not fastnumbers.isreal(x)


@given(text(min_size=2))
def test_isreal_given_unicode_of_more_than_one_char_returns_False(x):
    assume(not a_number(x))
    assert not fastnumbers.isreal(x)


@given(text() | binary())
@example('+')
@example('-')
@example('e')
@example('e8')
@example('.')
def test_isreal_returns_False_if_given_non_number_string(x):
    assume(not a_number(x))
    assert not fastnumbers.isreal(x)


def test_isreal_returns_False_for_nan_string_unless_allow_nan_is_True():
    assert not fastnumbers.isreal('nan')
    assert fastnumbers.isreal('nan', allow_nan=True)
    assert fastnumbers.isreal('-NaN', allow_nan=True)


def test_isreal_returns_False_for_inf_string_unless_allow_inf_is_True():
    assert not fastnumbers.isreal('inf')
    assert fastnumbers.isreal('inf', allow_inf=True)
    assert fastnumbers.isreal('-INFINITY', allow_inf=True)


############
# Is Float #
############


@given(integers())
def test_isfloat_returns_False_if_given_int(x):
    assert not fastnumbers.isfloat(x)


@given(floats())
def test_isfloat_returns_True_if_given_float(x):
    assert fastnumbers.isfloat(x)
    assert fastnumbers.isfloat(x, num_only=True)


@given(floats())
def test_isfloat_returns_False_if_given_float_and_str_only_is_True(x):
    assert not fastnumbers.isfloat(x, str_only=True)


@given(integers(), integers(0, 100), integers(0, 100))
@example(40992764608243448035, 1, 1)
@example(-41538374848935286698640072416676709, 1, 1)
@example(240278958776173358420034462324117625982, 1, 1)
@example(1609422692302207451978552816956662956486, 1, 1)
@example(-121799354242674784350540853922878239740762834, 1, 1)
@example(32718704454132572934419741118153895444518280065843028297496525078, 1, 1)
@example(33684944745210074227862907273261282807602986571245071790093633147269, 1, 1)
def test_isfloat_returns_True_if_given_int_string_padded_or_not(x, y, z):
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert fastnumbers.isfloat(repr(x))
    assert fastnumbers.isfloat(repr(x), str_only=True)
    assert fastnumbers.isfloat(y)


@given(floats(), integers(0, 100), integers(0, 100))
def test_isfloat_returns_True_if_given_float_string_padded_or_not(x, y, z):
    assume(not math.isnan(x))
    assume(not math.isinf(x))
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert fastnumbers.isfloat(repr(x))
    assert fastnumbers.isfloat(repr(x), str_only=True)
    assert fastnumbers.isfloat(y)


@given(floats())
def test_isfloat_returns_False_if_given_string_and_num_only_is_True(x):
    assume(not math.isnan(x))
    assume(not math.isinf(x))
    assert not fastnumbers.isfloat(repr(x), num_only=True)


@given(sampled_from(digits))
def test_isfloat_given_unicode_digit_returns_True(x):
    assert fastnumbers.isfloat(x)
    # Try padded as well
    assert fastnumbers.isfloat(u'   ' + x + u'   ')


@given(sampled_from(numeric_not_digit_not_int))
def test_isfloat_given_unicode_numeral_returns_True(x):
    assert fastnumbers.isfloat(x)
    # Try padded as well
    assert fastnumbers.isfloat(u'   ' + x + u'   ')


@given(sampled_from(not_numeric))
def test_isfloat_given_unicode_non_numeral_returns_False(x):
    assert not fastnumbers.isfloat(x)


@given(text(min_size=2))
def test_isfloat_given_unicode_of_more_than_one_char_returns_False(x):
    assume(not a_number(x))
    assert not fastnumbers.isfloat(x)


@given(text() | binary())
@example('+')
@example('-')
@example('e')
@example('e8')
@example('.')
def test_isfloat_returns_False_if_given_non_number_string(x):
    assume(not a_number(x))
    assert not fastnumbers.isfloat(x)


def test_isfloat_returns_False_for_nan_string_unless_allow_nan_is_True():
    assert not fastnumbers.isfloat('nan')
    assert fastnumbers.isfloat('nan', allow_nan=True)
    assert fastnumbers.isfloat('-NaN', allow_nan=True)


def test_isfloat_returns_False_for_inf_string_unless_allow_inf_is_True():
    assert not fastnumbers.isfloat('inf')
    assert fastnumbers.isfloat('inf', allow_inf=True)
    assert fastnumbers.isfloat('-INFINITY', allow_inf=True)


##########
# Is Int #
##########


@given(integers())
def test_isint_returns_True_if_given_int(x):
    assert fastnumbers.isint(x, num_only=True)


@given(floats())
def test_isint_returns_False_if_given_float(x):
    assert not fastnumbers.isint(x)


@given(integers())
def test_isint_returns_False_if_given_int_and_str_only_is_True(x):
    assert not fastnumbers.isint(x, str_only=True)


@given(integers(), integers(0, 100), integers(0, 100))
@example(40992764608243448035, 1, 1)
@example(-41538374848935286698640072416676709, 1, 1)
@example(240278958776173358420034462324117625982, 1, 1)
@example(1609422692302207451978552816956662956486, 1, 1)
@example(-121799354242674784350540853922878239740762834, 1, 1)
@example(32718704454132572934419741118153895444518280065843028297496525078, 1, 1)
@example(33684944745210074227862907273261282807602986571245071790093633147269, 1, 1)
def test_isint_returns_True_if_given_int_string_padded_or_not(x, y, z):
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert fastnumbers.isint(repr(x)) is True
    assert fastnumbers.isint(repr(x), str_only=True)
    assert fastnumbers.isint(y)
    for base in range(2, 36 + 1):
        if len(repr(x)) < 30:  # Avoid recursion error because of overly simple baseN function.
            assert fastnumbers.isint(baseN(x, base), base=base)
    assert fastnumbers.isint(bin(x), base=2)
    assert fastnumbers.isint(bin(x), base=0)
    assert fastnumbers.isint(oct(x), base=8)
    assert fastnumbers.isint(oct(x), base=0)
    if python_version_tuple()[0] == '2':
        assert fastnumbers.isint(oct(x).replace('0o', '0'), base=8)
        assert fastnumbers.isint(oct(x).replace('0o', '0'), base=0)
    assert fastnumbers.isint(hex(x), base=16)
    assert fastnumbers.isint(hex(x), base=0)


@given(floats(), integers(0, 100), integers(0, 100))
def test_isint_returns_False_if_given_float_string_padded_or_not(x, y, z):
    assume(not math.isnan(x))
    assume(not math.isinf(x))
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert not fastnumbers.isint(repr(x))
    assert not fastnumbers.isint(y)
    for base in range(2, 36 + 1):
        if len(y) < 30:  # Avoid recursion error because of overly simple baseN function.
            assert not fastnumbers.isint(y, base=base)


@given(integers())
def test_isint_returns_False_if_given_string_and_num_only_is_True(x):
    assert not fastnumbers.isint(repr(x), num_only=True)


@given(sampled_from(digits))
def test_isint_given_unicode_digit_returns_True(x):
    assert fastnumbers.isint(x)
    # Try padded as well
    assert fastnumbers.isint(u'   ' + x + u'   ')


@given(sampled_from(numeric_not_digit_not_int))
def test_isint_given_unicode_numeral_returns_False(x):
    assert not fastnumbers.isint(x)


@given(sampled_from(not_numeric))
def test_isint_given_unicode_non_numeral_returns_False(x):
    assert not fastnumbers.isint(x)


@given(text(min_size=2))
def test_isint_given_unicode_of_more_than_one_char_returns_False(x):
    assume(not a_number(x))
    assert not fastnumbers.isint(x)


@given(text() | binary())
@example('+')
@example('-')
@example('e')
@example('e8')
@example('.')
def test_isint_returns_False_if_given_non_number_string(x):
    assume(not a_number(x))
    assert not fastnumbers.isint(x)


def test_isint_returns_False_for_nan_or_inf_string():
    assert not fastnumbers.isint('nan')
    assert not fastnumbers.isint('inf')


###############
# Is Int-Like #
###############


@given(integers())
def test_isintlike_returns_True_if_given_int(x):
    assert fastnumbers.isintlike(x)
    assert fastnumbers.isintlike(x, num_only=True)


@given(floats())
def test_isintlike_returns_False_if_given_non_integer_float(x):
    assume(not x.is_integer())
    assert not fastnumbers.isintlike(x)


@given(floats())
def test_isintlike_returns_True_if_given_integer_float(x):
    assume(x.is_integer())
    assert fastnumbers.isintlike(x)


@given(integers())
def test_isintlike_returns_False_if_given_int_and_str_only_is_True(x):
    assert not fastnumbers.isintlike(x, str_only=True)


@given(floats())
def test_isintlike_returns_False_if_given_integer_float_and_str_only_is_True(x):
    assume(x.is_integer())
    assert not fastnumbers.isintlike(x, str_only=True)


@given(integers(), integers(0, 100), integers(0, 100))
@example(40992764608243448035, 1, 1)
@example(-41538374848935286698640072416676709, 1, 1)
@example(240278958776173358420034462324117625982, 1, 1)
@example(1609422692302207451978552816956662956486, 1, 1)
@example(-121799354242674784350540853922878239740762834, 1, 1)
@example(32718704454132572934419741118153895444518280065843028297496525078, 1, 1)
@example(33684944745210074227862907273261282807602986571245071790093633147269, 1, 1)
def test_isintlike_returns_True_if_given_int_string_padded_or_not(x, y, z):
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert fastnumbers.isintlike(repr(x))
    assert fastnumbers.isintlike(repr(x), str_only=True)
    assert fastnumbers.isintlike(y)


@given(floats(), integers(0, 100), integers(0, 100))
def test_isintlike_returns_True_if_given_integer_float_string_padded_or_not(x, y, z):
    assume(not math.isnan(x))
    assume(not math.isinf(x))
    assume(x.is_integer())
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert fastnumbers.isintlike(repr(x))
    assert fastnumbers.isintlike(y)


@given(floats(), integers(0, 100), integers(0, 100))
def test_isintlike_returns_False_if_given_non_integer_float_string_padded_or_not(x, y, z):
    assume(not math.isnan(x))
    assume(not math.isinf(x))
    assume(not x.is_integer())
    y = ''.join(repeat(' ', y)) + repr(x) + ''.join(repeat(' ', z))
    assert not fastnumbers.isintlike(repr(x))
    assert not fastnumbers.isintlike(y)


@given(integers())
def test_isintlike_returns_False_if_given_string_and_num_only_is_True(x):
    assert not fastnumbers.isintlike(repr(x), num_only=True)


@given(sampled_from(digits))
def test_isintlike_given_unicode_digit_returns_True(x):
    assert fastnumbers.isintlike(x)
    # Try padded as well
    assert fastnumbers.isintlike(u'   ' + x + u'   ')


@given(sampled_from(numeric_not_digit_not_int))
def test_isintlike_given_unicode_non_digit_numeral_returns_False(x):
    assert not fastnumbers.isintlike(x)


@given(sampled_from(numeric_not_digit))
def test_isintlike_given_unicode_digit_numeral_returns_False(x):
    assume(unicodedata.numeric(x).is_integer())
    assert fastnumbers.isintlike(x)
    # Try padded as well
    assert fastnumbers.isintlike(u'   ' + x + u'   ')


@given(sampled_from(not_numeric))
def test_isintlike_given_unicode_non_numeral_returns_False(x):
    assert not fastnumbers.isintlike(x)


@given(text(min_size=2))
def test_isintlike_given_unicode_of_more_than_one_char_returns_False(x):
    assume(not a_number(x))
    assert not fastnumbers.isintlike(x)


@given(text() | binary())
@example('+')
@example('-')
@example('e')
@example('e8')
@example('.')
def test_isintlike_returns_False_if_given_non_number_string(x):
    assume(not a_number(x))
    assert not fastnumbers.isintlike(x)


def test_isintlike_returns_False_for_nan_or_inf_string():
    assert not fastnumbers.isintlike('nan')
    assert not fastnumbers.isintlike('inf')



from __future__ import print_function, division

# Local import
from time_base import comparison_timer

forceint_re = '''\
import re
float_regex = re.compile(r'[-+]?\d*\.?\d+(?:[eE][-+]?\d+)?$')
float_match = float_regex.match
int_regex = re.compile(r'[-+]\d+$')
int_match = int_regex.match
def forceint_re(x):
    """Function to simulate safe_forceint but with regular expressions."""
    try:
        if int_match(x):
            return int(x)
        elif float_match(x):
            return int(float(x))
        else:
            return x
    except TypeError:
        return int(x)
'''

forceint_try = '''\
def forceint_try(x):
    """Function to simulate safe_forceint but with try/except."""
    try:
        return int(x)
    except ValueError:
        try:
            return int(float(x))
        except ValueError:
            return x
'''

comparison_timer(['forceint_re', forceint_re],
                 ['forceint_try', forceint_try],
                 ['fast_forceint', 'from fastnumbers import fast_forceint'])



import sys

# Load test runner from correct python version.
if sys.version_info[0:2] == (2, 7):
    from builtin_tests.py27.builtin_test_float import *
    from builtin_tests.py27.builtin_test_int import *
elif sys.version_info[0:2] == (3, 3):
    from builtin_tests.py33.builtin_test_float import *
    from builtin_tests.py33.builtin_test_int import *
elif sys.version_info[0:2] == (3, 4):
    from builtin_tests.py34.builtin_test_float import *
    from builtin_tests.py34.builtin_test_int import *
elif sys.version_info[0:2] == (3, 5):
    from builtin_tests.py35.builtin_test_float import *
    from builtin_tests.py35.builtin_test_int import *
else:
    from builtin_tests.py36.builtin_test_float import *
    from builtin_tests.py36.builtin_test_int import *



from __future__ import print_function, division

# Local import
from time_base import comparison_timer

isreal_re = '''\
import re
real_regex = re.compile(r'[-+]?\d*\.?\d+(?:[eE][-+]?\d+)?$')
real_match = real_regex.match
nums = set([float, int])
def isreal_re(x):
    """Function to simulate isreal but with regular expressions."""
    return type(x) in nums or bool(real_match(x))
'''

isreal_try = '''\
def isreal_try(x):
    """Function to simulate isreal but with try/except."""
    try:
        float(x)
    except ValueError:
        return False
    else:
        return True
'''

comparison_timer(['isreal_re', isreal_re],
                 ['isreal_try', isreal_try],
                 ['isreal', 'from fastnumbers import isreal'])



from __future__ import print_function, division

# Local import
from time_base import comparison_timer

real_re = '''\
import re
real_regex = re.compile(r'[-+]?\d*\.?\d+(?:[eE][-+]?\d+)?$')
real_match = real_regex.match
int_regex = re.compile(r'[-+]?\d+$')
int_match = int_regex.match
def real_re(x):
    """Function to simulate safe_real but with regular expressions."""
    try:
        if int_match(x):
            return int(x)
        elif real_match(x):
            return float(x)
        else:
            return x
    except TypeError:
        if type(x) in set([float, int]):
            return x
        else:
            raise TypeError
'''

real_try = '''\
def real_try(x):
    """Function to simulate safe_real but with try/except."""
    try:
        a = float(x)
    except ValueError:
        return x
    else:
        b = int(a)
        return b if a == b else b
'''

comparison_timer(['real_re', real_re],
                 ['real_try', real_try],
                 ['fast_real', 'from fastnumbers import fast_real'])



from __future__ import print_function, division

# Local import
from time_base import comparison_timer

isfloat_re = '''\
import re
float_regex = re.compile(r'[-+]?\d*\.?\d+(?:[eE][-+]?\d+)?$')
float_match = float_regex.match
nums = set([float, int])
def isfloat_re(x):
    """Function to simulate isfloat but with regular expressions."""
    t = type(x)
    return t == float if t in nums else bool(float_match(x))
'''

isfloat_try = '''\
def isfloat_try(x):
    """Function to simulate isfloat but with try/except."""
    try:
        float(x)
    except ValueError:
        return False
    else:
        return type(x) != int
'''

comparison_timer(['isfloat_re', isfloat_re],
                 ['isfloat_try', isfloat_try],
                 ['isfloat', 'from fastnumbers import isfloat'])



from __future__ import print_function, division

# Local import
from time_base import comparison_timer

int_re = '''\
import re
int_regex = re.compile(r'[-+]?\d+$')
int_match = int_regex.match
def int_re(x):
    """Function to simulate safe_int but with regular expressions."""
    try:
        if int_match(x):
            return int(x)
        else:
            return x
    except TypeError:
        return int(x)
'''

int_try = '''\
def int_try(x):
    """Function to simulate safe_int but with try/except."""
    try:
        return int(x)
    except ValueError:
        return x
'''

comparison_timer(['int_re', int_re],
                 ['int_try', int_try],
                 ['fast_int', 'from fastnumbers import fast_int'])



from __future__ import print_function, division

import sys
from timeit import repeat
from time_base import mean


def time_input_against_all_functions(value, label):
    """Run the given input on all function types."""
    funcs = ('fast_real', 'fast_float', 'fast_int', 'fast_forceint',
             'isreal', 'isfloat', 'isint', 'isintlike',)
    fmt = '{func}({value!r})'
    for func in funcs:
        print(label + ',', func + ':', end=' ')
        sys.stdout.flush()
        time_results = repeat(fmt.format(func=func, value=value),
                              'from fastnumbers import {}'.format(func),
                              repeat=10)
        time_results = mean(time_results)
        print(time_results, 'seconds')
    print()


print('All timing results are the average of 10 runs.')
print()
time_input_against_all_functions('not_a_number', 'Non-number String')
time_input_against_all_functions('-41053', 'Int String')
time_input_against_all_functions('35892482945872302493947939485729', 'Large Int String')
time_input_against_all_functions('-41053.543034e34', 'Float String')
time_input_against_all_functions('-41053.543028758302e256', 'Large Float String')
time_input_against_all_functions(-41053.543028758302e100, 'Float')
time_input_against_all_functions(-41053, 'Int')



from __future__ import print_function, division

# Local import
from time_base import comparison_timer

isintlike_re = '''\
import re
float_regex = re.compile(r'[-+]?\d*\.?\d+(?:[eE][-+]?\d+)?$')
float_match = float_regex.match
int_regex = re.compile(r'[-+]?\d+$')
int_match = int_regex.match
def isintlike_re(x):
    """Function to simulate isintlike but with regular expressions."""
    try:
        if int_match(x):
            return True
        elif float_match(x):
            return float(x).is_integer()
        else:
            return False
    except TypeError:
        return int(x) == x
'''

isintlike_try = '''\
def isintlike_try(x):
    """Function to simulate isintlike but with try/except."""
    try:
        a = int(x)
    except ValueError:
        try:
            a = float(x)
        except ValueError:
            return False
        else:
            return a.is_integer()
    else:
        return a == float(x)
'''

comparison_timer(['isintlike_re', isintlike_re],
                 ['isintlike_try', isintlike_try],
                 ['isintlike', 'from fastnumbers import isintlike'])



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Std lib imports
import re
import glob
from os.path import join, abspath

# Non-std lib imports
from setuptools import setup, Extension


DESCRIPTION = "Super-fast and clean conversions to numbers."
try:
    with open('README.rst') as fl:
        LONG_DESCRIPTION = fl.read()
except IOError:
    LONG_DESCRIPTION = DESCRIPTION


def current_version():
    '''Get the current version number'''
    VERSIONFILE = join('include', 'version.h')
    with open(VERSIONFILE, "rt") as fl:
        versionstring = fl.read()
    m = re.search(r"#define \w+_VERSION \"(.*)\"", versionstring)
    if m:
        return m.group(1)
    else:
        s = "Unable to locate version string in {0}"
        raise RuntimeError(s.format(VERSIONFILE))


# Extension definition
ext = Extension('fastnumbers', glob.glob('src/*.c'),
                include_dirs=[abspath(join('include'))],
                extra_compile_args=[])


# Define the build
setup(name='fastnumbers',
      version=current_version(),
      author='Seth M. Morton',
      author_email='drtuba78@gmail.com',
      url='https://github.com/SethMMorton/fastnumbers',
      license='MIT',
      ext_modules=[ext],
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION,
      classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Financial and Insurance Industry',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Utilities',
        'Topic :: Text Processing',
        'Topic :: Text Processing :: Filters',
    )
)




import unittest, struct
import os
import sys
import builtin_tests.py33.support as support
from math import isinf, isnan, copysign, ldexp
import operator
import random, fractions
from fastnumbers import float
import builtins

INF = float("inf")
NAN = float("nan")

have_getformat = hasattr(float, "__getformat__")
requires_getformat = unittest.skipUnless(have_getformat,
                                         "requires __getformat__")
requires_setformat = unittest.skipUnless(hasattr(float, "__setformat__"),
                                         "requires __setformat__")

#locate file with float format test values
test_dir = os.path.dirname(__file__) or os.curdir
# format_testfile = os.path.join(test_dir, 'formatfloat_testcases.txt')

class GeneralFloatCases(unittest.TestCase):

    def test_float(self):
        self.assertEqual(float(3.14), 3.14)
        self.assertEqual(float(314), 314.0)
        self.assertEqual(float("  3.14  "), 3.14)
        self.assertEqual(float(b" 3.14  "), 3.14)
        self.assertRaises(ValueError, float, "  0x3.1  ")
        self.assertRaises(ValueError, float, "  -0x3.p-1  ")
        self.assertRaises(ValueError, float, "  +0x3.p-1  ")
        self.assertRaises(ValueError, float, "++3.14")
        self.assertRaises(ValueError, float, "+-3.14")
        self.assertRaises(ValueError, float, "-+3.14")
        self.assertRaises(ValueError, float, "--3.14")
        self.assertRaises(ValueError, float, ".nan")
        self.assertRaises(ValueError, float, "+.inf")
        self.assertRaises(ValueError, float, ".")
        self.assertRaises(ValueError, float, "-.")
        self.assertRaises(ValueError, float, b"-")
        self.assertRaises(TypeError, float, {})
        # Lone surrogate
        # self.assertRaises(UnicodeEncodeError, float, '\uD8F0')
        # check that we don't accept alternate exponent markers
        self.assertRaises(ValueError, float, "-1.7d29")
        self.assertRaises(ValueError, float, "3D-14")
        self.assertEqual(float("  \u0663.\u0661\u0664  "), 3.14)
        self.assertEqual(float("\N{EM SPACE}3.14\N{EN SPACE}"), 3.14)
        # extra long strings should not be a problem
        float(b'.' + b'1'*1000)
        float('.' + '1'*1000)

    def test_error_message(self):
        testlist = ('\xbd', '123\xbd', '  123 456  ')
        for s in testlist:
            try:
                float(s)
            except ValueError as e:
                self.assertIn(s.strip(), e.args[0])
            else:
                self.fail("Expected int(%r) to raise a ValueError", s)


    @support.run_with_locale('LC_NUMERIC', 'fr_FR', 'de_DE')
    def test_float_with_comma(self):
        # set locale to something that doesn't use '.' for the decimal point
        # float must not accept the locale specific decimal point but
        # it still has to accept the normal python syntax
        import locale
        if not locale.localeconv()['decimal_point'] == ',':
            self.skipTest('decimal_point is not ","')

        self.assertEqual(float("  3.14  "), 3.14)
        self.assertEqual(float("+3.14  "), 3.14)
        self.assertEqual(float("-3.14  "), -3.14)
        self.assertEqual(float(".14  "), .14)
        self.assertEqual(float("3.  "), 3.0)
        self.assertEqual(float("3.e3  "), 3000.0)
        self.assertEqual(float("3.2e3  "), 3200.0)
        self.assertEqual(float("2.5e-1  "), 0.25)
        self.assertEqual(float("5e-1"), 0.5)
        self.assertRaises(ValueError, float, "  3,14  ")
        self.assertRaises(ValueError, float, "  +3,14  ")
        self.assertRaises(ValueError, float, "  -3,14  ")
        self.assertRaises(ValueError, float, "  0x3.1  ")
        self.assertRaises(ValueError, float, "  -0x3.p-1  ")
        self.assertRaises(ValueError, float, "  +0x3.p-1  ")
        self.assertEqual(float("  25.e-1  "), 2.5)
        self.assertAlmostEqual(float("  .25e-1  "), .025)

    def test_floatconversion(self):
        # Make sure that calls to __float__() work properly
        class Foo0:
            def __float__(self):
                return 42.

        class Foo1(object):
            def __float__(self):
                return 42.

        class Foo2(builtins.float):
            def __float__(self):
                return 42.

        class Foo3(builtins.float):
            def __new__(cls, value=0.):
                return builtins.float.__new__(cls, 2*value)

            def __float__(self):
                return self

        class Foo4(builtins.float):
            def __float__(self):
                return 42

        # Issue 5759: __float__ not called on str subclasses (though it is on
        # unicode subclasses).
        class FooStr(str):
            def __float__(self):
                return float(str(self)) + 1

        self.assertAlmostEqual(float(Foo0()), 42.)
        self.assertAlmostEqual(float(Foo1()), 42.)
        self.assertAlmostEqual(float(Foo2()), 42.)
        self.assertAlmostEqual(float(Foo3(21)), 42.)
        self.assertRaises(TypeError, float, Foo4(42))
        self.assertAlmostEqual(float(FooStr('8')), 9.)

#     def test_is_integer(self):
#         self.assertFalse((1.1).is_integer())
#         self.assertTrue((1.).is_integer())
#         self.assertFalse(float("nan").is_integer())
#         self.assertFalse(float("inf").is_integer())
#
#     def test_floatasratio(self):
#         for f, ratio in [
#                 (0.875, (7, 8)),
#                 (-0.875, (-7, 8)),
#                 (0.0, (0, 1)),
#                 (11.5, (23, 2)),
#             ]:
#             self.assertEqual(f.as_integer_ratio(), ratio)
#
#         for i in range(10000):
#             f = random.random()
#             f *= 10 ** random.randint(-100, 100)
#             n, d = f.as_integer_ratio()
#             self.assertEqual(float(n).__truediv__(d), f)
#
#         R = fractions.Fraction
#         self.assertEqual(R(0, 1),
#                          R(*float(0.0).as_integer_ratio()))
#         self.assertEqual(R(5, 2),
#                          R(*float(2.5).as_integer_ratio()))
#         self.assertEqual(R(1, 2),
#                          R(*float(0.5).as_integer_ratio()))
#         self.assertEqual(R(4728779608739021, 2251799813685248),
#                          R(*float(2.1).as_integer_ratio()))
#         self.assertEqual(R(-4728779608739021, 2251799813685248),
#                          R(*float(-2.1).as_integer_ratio()))
#         self.assertEqual(R(-2100, 1),
#                          R(*float(-2100.0).as_integer_ratio()))
#
#         self.assertRaises(OverflowError, float('inf').as_integer_ratio)
#         self.assertRaises(OverflowError, float('-inf').as_integer_ratio)
#         self.assertRaises(ValueError, float('nan').as_integer_ratio)
#
#     def test_float_containment(self):
#         floats = (INF, -INF, 0.0, 1.0, NAN)
#         for f in floats:
#             self.assertIn(f, [f])
#             self.assertIn(f, (f,))
#             self.assertIn(f, {f})
#             self.assertIn(f, {f: None})
#             self.assertEqual([f].count(f), 1, "[].count('%r') != 1" % f)
#             self.assertIn(f, floats)
#
#         for f in floats:
#             # nonidentical containers, same type, same contents
#             self.assertTrue([f] == [f], "[%r] != [%r]" % (f, f))
#             self.assertTrue((f,) == (f,), "(%r,) != (%r,)" % (f, f))
#             self.assertTrue({f} == {f}, "{%r} != {%r}" % (f, f))
#             self.assertTrue({f : None} == {f: None}, "{%r : None} != "
#                                                    "{%r : None}" % (f, f))
#
#             # identical containers
#             l, t, s, d = [f], (f,), {f}, {f: None}
#             self.assertTrue(l == l, "[%r] not equal to itself" % f)
#             self.assertTrue(t == t, "(%r,) not equal to itself" % f)
#             self.assertTrue(s == s, "{%r} not equal to itself" % f)
#             self.assertTrue(d == d, "{%r : None} not equal to itself" % f)
#
#     def assertEqualAndEqualSign(self, a, b):
#         # fail unless a == b and a and b have the same sign bit;
#         # the only difference from assertEqual is that this test
#         # distinguishes -0.0 and 0.0.
#         self.assertEqual((a, copysign(1.0, a)), (b, copysign(1.0, b)))
#
#     @support.requires_IEEE_754
#     def test_float_mod(self):
#         # Check behaviour of % operator for IEEE 754 special cases.
#         # In particular, check signs of zeros.
#         mod = operator.mod
#
#         self.assertEqualAndEqualSign(mod(-1.0, 1.0), 0.0)
#         self.assertEqualAndEqualSign(mod(-1e-100, 1.0), 1.0)
#         self.assertEqualAndEqualSign(mod(-0.0, 1.0), 0.0)
#         self.assertEqualAndEqualSign(mod(0.0, 1.0), 0.0)
#         self.assertEqualAndEqualSign(mod(1e-100, 1.0), 1e-100)
#         self.assertEqualAndEqualSign(mod(1.0, 1.0), 0.0)
#
#         self.assertEqualAndEqualSign(mod(-1.0, -1.0), -0.0)
#         self.assertEqualAndEqualSign(mod(-1e-100, -1.0), -1e-100)
#         self.assertEqualAndEqualSign(mod(-0.0, -1.0), -0.0)
#         self.assertEqualAndEqualSign(mod(0.0, -1.0), -0.0)
#         self.assertEqualAndEqualSign(mod(1e-100, -1.0), -1.0)
#         self.assertEqualAndEqualSign(mod(1.0, -1.0), -0.0)
#
#     @support.requires_IEEE_754
#     def test_float_pow(self):
#         # test builtin pow and ** operator for IEEE 754 special cases.
#         # Special cases taken from section F.9.4.4 of the C99 specification
#
#         for pow_op in pow, operator.pow:
#             # x**NAN is NAN for any x except 1
#             self.assertTrue(isnan(pow_op(-INF, NAN)))
#             self.assertTrue(isnan(pow_op(-2.0, NAN)))
#             self.assertTrue(isnan(pow_op(-1.0, NAN)))
#             self.assertTrue(isnan(pow_op(-0.5, NAN)))
#             self.assertTrue(isnan(pow_op(-0.0, NAN)))
#             self.assertTrue(isnan(pow_op(0.0, NAN)))
#             self.assertTrue(isnan(pow_op(0.5, NAN)))
#             self.assertTrue(isnan(pow_op(2.0, NAN)))
#             self.assertTrue(isnan(pow_op(INF, NAN)))
#             self.assertTrue(isnan(pow_op(NAN, NAN)))
#
#             # NAN**y is NAN for any y except +-0
#             self.assertTrue(isnan(pow_op(NAN, -INF)))
#             self.assertTrue(isnan(pow_op(NAN, -2.0)))
#             self.assertTrue(isnan(pow_op(NAN, -1.0)))
#             self.assertTrue(isnan(pow_op(NAN, -0.5)))
#             self.assertTrue(isnan(pow_op(NAN, 0.5)))
#             self.assertTrue(isnan(pow_op(NAN, 1.0)))
#             self.assertTrue(isnan(pow_op(NAN, 2.0)))
#             self.assertTrue(isnan(pow_op(NAN, INF)))
#
#             # (+-0)**y raises ZeroDivisionError for y a negative odd integer
#             self.assertRaises(ZeroDivisionError, pow_op, -0.0, -1.0)
#             self.assertRaises(ZeroDivisionError, pow_op, 0.0, -1.0)
#
#             # (+-0)**y raises ZeroDivisionError for y finite and negative
#             # but not an odd integer
#             self.assertRaises(ZeroDivisionError, pow_op, -0.0, -2.0)
#             self.assertRaises(ZeroDivisionError, pow_op, -0.0, -0.5)
#             self.assertRaises(ZeroDivisionError, pow_op, 0.0, -2.0)
#             self.assertRaises(ZeroDivisionError, pow_op, 0.0, -0.5)
#
#             # (+-0)**y is +-0 for y a positive odd integer
#             self.assertEqualAndEqualSign(pow_op(-0.0, 1.0), -0.0)
#             self.assertEqualAndEqualSign(pow_op(0.0, 1.0), 0.0)
#
#             # (+-0)**y is 0 for y finite and positive but not an odd integer
#             self.assertEqualAndEqualSign(pow_op(-0.0, 0.5), 0.0)
#             self.assertEqualAndEqualSign(pow_op(-0.0, 2.0), 0.0)
#             self.assertEqualAndEqualSign(pow_op(0.0, 0.5), 0.0)
#             self.assertEqualAndEqualSign(pow_op(0.0, 2.0), 0.0)
#
#             # (-1)**+-inf is 1
#             self.assertEqualAndEqualSign(pow_op(-1.0, -INF), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-1.0, INF), 1.0)
#
#             # 1**y is 1 for any y, even if y is an infinity or nan
#             self.assertEqualAndEqualSign(pow_op(1.0, -INF), 1.0)
#             self.assertEqualAndEqualSign(pow_op(1.0, -2.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(1.0, -1.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(1.0, -0.5), 1.0)
#             self.assertEqualAndEqualSign(pow_op(1.0, -0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(1.0, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(1.0, 0.5), 1.0)
#             self.assertEqualAndEqualSign(pow_op(1.0, 1.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(1.0, 2.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(1.0, INF), 1.0)
#             self.assertEqualAndEqualSign(pow_op(1.0, NAN), 1.0)
#
#             # x**+-0 is 1 for any x, even if x is a zero, infinity, or nan
#             self.assertEqualAndEqualSign(pow_op(-INF, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-2.0, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-1.0, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-0.5, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-0.0, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(0.0, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(0.5, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(1.0, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(2.0, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(INF, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(NAN, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-INF, -0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-2.0, -0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-1.0, -0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-0.5, -0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-0.0, -0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(0.0, -0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(0.5, -0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(1.0, -0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(2.0, -0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(INF, -0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(NAN, -0.0), 1.0)
#
#             # x**y defers to complex pow for finite negative x and
#             # non-integral y.
#             self.assertEqual(type(pow_op(-2.0, -0.5)), complex)
#             self.assertEqual(type(pow_op(-2.0, 0.5)), complex)
#             self.assertEqual(type(pow_op(-1.0, -0.5)), complex)
#             self.assertEqual(type(pow_op(-1.0, 0.5)), complex)
#             self.assertEqual(type(pow_op(-0.5, -0.5)), complex)
#             self.assertEqual(type(pow_op(-0.5, 0.5)), complex)
#
#             # x**-INF is INF for abs(x) < 1
#             self.assertEqualAndEqualSign(pow_op(-0.5, -INF), INF)
#             self.assertEqualAndEqualSign(pow_op(-0.0, -INF), INF)
#             self.assertEqualAndEqualSign(pow_op(0.0, -INF), INF)
#             self.assertEqualAndEqualSign(pow_op(0.5, -INF), INF)
#
#             # x**-INF is 0 for abs(x) > 1
#             self.assertEqualAndEqualSign(pow_op(-INF, -INF), 0.0)
#             self.assertEqualAndEqualSign(pow_op(-2.0, -INF), 0.0)
#             self.assertEqualAndEqualSign(pow_op(2.0, -INF), 0.0)
#             self.assertEqualAndEqualSign(pow_op(INF, -INF), 0.0)
#
#             # x**INF is 0 for abs(x) < 1
#             self.assertEqualAndEqualSign(pow_op(-0.5, INF), 0.0)
#             self.assertEqualAndEqualSign(pow_op(-0.0, INF), 0.0)
#             self.assertEqualAndEqualSign(pow_op(0.0, INF), 0.0)
#             self.assertEqualAndEqualSign(pow_op(0.5, INF), 0.0)
#
#             # x**INF is INF for abs(x) > 1
#             self.assertEqualAndEqualSign(pow_op(-INF, INF), INF)
#             self.assertEqualAndEqualSign(pow_op(-2.0, INF), INF)
#             self.assertEqualAndEqualSign(pow_op(2.0, INF), INF)
#             self.assertEqualAndEqualSign(pow_op(INF, INF), INF)
#
#             # (-INF)**y is -0.0 for y a negative odd integer
#             self.assertEqualAndEqualSign(pow_op(-INF, -1.0), -0.0)
#
#             # (-INF)**y is 0.0 for y negative but not an odd integer
#             self.assertEqualAndEqualSign(pow_op(-INF, -0.5), 0.0)
#             self.assertEqualAndEqualSign(pow_op(-INF, -2.0), 0.0)
#
#             # (-INF)**y is -INF for y a positive odd integer
#             self.assertEqualAndEqualSign(pow_op(-INF, 1.0), -INF)
#
#             # (-INF)**y is INF for y positive but not an odd integer
#             self.assertEqualAndEqualSign(pow_op(-INF, 0.5), INF)
#             self.assertEqualAndEqualSign(pow_op(-INF, 2.0), INF)
#
#             # INF**y is INF for y positive
#             self.assertEqualAndEqualSign(pow_op(INF, 0.5), INF)
#             self.assertEqualAndEqualSign(pow_op(INF, 1.0), INF)
#             self.assertEqualAndEqualSign(pow_op(INF, 2.0), INF)
#
#             # INF**y is 0.0 for y negative
#             self.assertEqualAndEqualSign(pow_op(INF, -2.0), 0.0)
#             self.assertEqualAndEqualSign(pow_op(INF, -1.0), 0.0)
#             self.assertEqualAndEqualSign(pow_op(INF, -0.5), 0.0)
#
#             # basic checks not covered by the special cases above
#             self.assertEqualAndEqualSign(pow_op(-2.0, -2.0), 0.25)
#             self.assertEqualAndEqualSign(pow_op(-2.0, -1.0), -0.5)
#             self.assertEqualAndEqualSign(pow_op(-2.0, -0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-2.0, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-2.0, 1.0), -2.0)
#             self.assertEqualAndEqualSign(pow_op(-2.0, 2.0), 4.0)
#             self.assertEqualAndEqualSign(pow_op(-1.0, -2.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-1.0, -1.0), -1.0)
#             self.assertEqualAndEqualSign(pow_op(-1.0, -0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-1.0, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-1.0, 1.0), -1.0)
#             self.assertEqualAndEqualSign(pow_op(-1.0, 2.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(2.0, -2.0), 0.25)
#             self.assertEqualAndEqualSign(pow_op(2.0, -1.0), 0.5)
#             self.assertEqualAndEqualSign(pow_op(2.0, -0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(2.0, 0.0), 1.0)
#             self.assertEqualAndEqualSign(pow_op(2.0, 1.0), 2.0)
#             self.assertEqualAndEqualSign(pow_op(2.0, 2.0), 4.0)
#
#             # 1 ** large and -1 ** large; some libms apparently
#             # have problems with these
#             self.assertEqualAndEqualSign(pow_op(1.0, -1e100), 1.0)
#             self.assertEqualAndEqualSign(pow_op(1.0, 1e100), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-1.0, -1e100), 1.0)
#             self.assertEqualAndEqualSign(pow_op(-1.0, 1e100), 1.0)
#
#             # check sign for results that underflow to 0
#             self.assertEqualAndEqualSign(pow_op(-2.0, -2000.0), 0.0)
#             self.assertEqual(type(pow_op(-2.0, -2000.5)), complex)
#             self.assertEqualAndEqualSign(pow_op(-2.0, -2001.0), -0.0)
#             self.assertEqualAndEqualSign(pow_op(2.0, -2000.0), 0.0)
#             self.assertEqualAndEqualSign(pow_op(2.0, -2000.5), 0.0)
#             self.assertEqualAndEqualSign(pow_op(2.0, -2001.0), 0.0)
#             self.assertEqualAndEqualSign(pow_op(-0.5, 2000.0), 0.0)
#             self.assertEqual(type(pow_op(-0.5, 2000.5)), complex)
#             self.assertEqualAndEqualSign(pow_op(-0.5, 2001.0), -0.0)
#             self.assertEqualAndEqualSign(pow_op(0.5, 2000.0), 0.0)
#             self.assertEqualAndEqualSign(pow_op(0.5, 2000.5), 0.0)
#             self.assertEqualAndEqualSign(pow_op(0.5, 2001.0), 0.0)
#
#             # check we don't raise an exception for subnormal results,
#             # and validate signs.  Tests currently disabled, since
#             # they fail on systems where a subnormal result from pow
#             # is flushed to zero (e.g. Debian/ia64.)
#             #self.assertTrue(0.0 < pow_op(0.5, 1048) < 1e-315)
#             #self.assertTrue(0.0 < pow_op(-0.5, 1048) < 1e-315)
#             #self.assertTrue(0.0 < pow_op(0.5, 1047) < 1e-315)
#             #self.assertTrue(0.0 > pow_op(-0.5, 1047) > -1e-315)
#             #self.assertTrue(0.0 < pow_op(2.0, -1048) < 1e-315)
#             #self.assertTrue(0.0 < pow_op(-2.0, -1048) < 1e-315)
#             #self.assertTrue(0.0 < pow_op(2.0, -1047) < 1e-315)
#             #self.assertTrue(0.0 > pow_op(-2.0, -1047) > -1e-315)
#
#
# @requires_setformat
# class FormatFunctionsTestCase(unittest.TestCase):
#
#     def setUp(self):
#         self.save_formats = {'double':float.__getformat__('double'),
#                              'float':float.__getformat__('float')}
#
#     def tearDown(self):
#         float.__setformat__('double', self.save_formats['double'])
#         float.__setformat__('float', self.save_formats['float'])
#
#     def test_getformat(self):
#         self.assertIn(float.__getformat__('double'),
#                       ['unknown', 'IEEE, big-endian', 'IEEE, little-endian'])
#         self.assertIn(float.__getformat__('float'),
#                       ['unknown', 'IEEE, big-endian', 'IEEE, little-endian'])
#         self.assertRaises(ValueError, float.__getformat__, 'chicken')
#         self.assertRaises(TypeError, float.__getformat__, 1)
#
#     def test_setformat(self):
#         for t in 'double', 'float':
#             float.__setformat__(t, 'unknown')
#             if self.save_formats[t] == 'IEEE, big-endian':
#                 self.assertRaises(ValueError, float.__setformat__,
#                                   t, 'IEEE, little-endian')
#             elif self.save_formats[t] == 'IEEE, little-endian':
#                 self.assertRaises(ValueError, float.__setformat__,
#                                   t, 'IEEE, big-endian')
#             else:
#                 self.assertRaises(ValueError, float.__setformat__,
#                                   t, 'IEEE, big-endian')
#                 self.assertRaises(ValueError, float.__setformat__,
#                                   t, 'IEEE, little-endian')
#             self.assertRaises(ValueError, float.__setformat__,
#                               t, 'chicken')
#         self.assertRaises(ValueError, float.__setformat__,
#                           'chicken', 'unknown')
#
# BE_DOUBLE_INF = b'\x7f\xf0\x00\x00\x00\x00\x00\x00'
# LE_DOUBLE_INF = bytes(reversed(BE_DOUBLE_INF))
# BE_DOUBLE_NAN = b'\x7f\xf8\x00\x00\x00\x00\x00\x00'
# LE_DOUBLE_NAN = bytes(reversed(BE_DOUBLE_NAN))
#
# BE_FLOAT_INF = b'\x7f\x80\x00\x00'
# LE_FLOAT_INF = bytes(reversed(BE_FLOAT_INF))
# BE_FLOAT_NAN = b'\x7f\xc0\x00\x00'
# LE_FLOAT_NAN = bytes(reversed(BE_FLOAT_NAN))
#
# # on non-IEEE platforms, attempting to unpack a bit pattern
# # representing an infinity or a NaN should raise an exception.
#
# @requires_setformat
# class UnknownFormatTestCase(unittest.TestCase):
#     def setUp(self):
#         self.save_formats = {'double':float.__getformat__('double'),
#                              'float':float.__getformat__('float')}
#         float.__setformat__('double', 'unknown')
#         float.__setformat__('float', 'unknown')
#
#     def tearDown(self):
#         float.__setformat__('double', self.save_formats['double'])
#         float.__setformat__('float', self.save_formats['float'])
#
#     def test_double_specials_dont_unpack(self):
#         for fmt, data in [('>d', BE_DOUBLE_INF),
#                           ('>d', BE_DOUBLE_NAN),
#                           ('<d', LE_DOUBLE_INF),
#                           ('<d', LE_DOUBLE_NAN)]:
#             self.assertRaises(ValueError, struct.unpack, fmt, data)
#
#     def test_float_specials_dont_unpack(self):
#         for fmt, data in [('>f', BE_FLOAT_INF),
#                           ('>f', BE_FLOAT_NAN),
#                           ('<f', LE_FLOAT_INF),
#                           ('<f', LE_FLOAT_NAN)]:
#             self.assertRaises(ValueError, struct.unpack, fmt, data)
#
#
# # on an IEEE platform, all we guarantee is that bit patterns
# # representing infinities or NaNs do not raise an exception; all else
# # is accident (today).
# # let's also try to guarantee that -0.0 and 0.0 don't get confused.
#
# class IEEEFormatTestCase(unittest.TestCase):
#
#     @support.requires_IEEE_754
#     def test_double_specials_do_unpack(self):
#         for fmt, data in [('>d', BE_DOUBLE_INF),
#                           ('>d', BE_DOUBLE_NAN),
#                           ('<d', LE_DOUBLE_INF),
#                           ('<d', LE_DOUBLE_NAN)]:
#             struct.unpack(fmt, data)
#
#     @support.requires_IEEE_754
#     def test_float_specials_do_unpack(self):
#         for fmt, data in [('>f', BE_FLOAT_INF),
#                           ('>f', BE_FLOAT_NAN),
#                           ('<f', LE_FLOAT_INF),
#                           ('<f', LE_FLOAT_NAN)]:
#             struct.unpack(fmt, data)
#
# class FormatTestCase(unittest.TestCase):
#
#     def test_format(self):
#         # these should be rewritten to use both format(x, spec) and
#         # x.__format__(spec)
#
#         self.assertEqual(format(0.0, 'f'), '0.000000')
#
#         # the default is 'g', except for empty format spec
#         self.assertEqual(format(0.0, ''), '0.0')
#         self.assertEqual(format(0.01, ''), '0.01')
#         self.assertEqual(format(0.01, 'g'), '0.01')
#
#         # empty presentation type should format in the same way as str
#         # (issue 5920)
#         x = 100/7.
#         self.assertEqual(format(x, ''), str(x))
#         self.assertEqual(format(x, '-'), str(x))
#         self.assertEqual(format(x, '>'), str(x))
#         self.assertEqual(format(x, '2'), str(x))
#
#         self.assertEqual(format(1.0, 'f'), '1.000000')
#
#         self.assertEqual(format(-1.0, 'f'), '-1.000000')
#
#         self.assertEqual(format( 1.0, ' f'), ' 1.000000')
#         self.assertEqual(format(-1.0, ' f'), '-1.000000')
#         self.assertEqual(format( 1.0, '+f'), '+1.000000')
#         self.assertEqual(format(-1.0, '+f'), '-1.000000')
#
#         # % formatting
#         self.assertEqual(format(-1.0, '%'), '-100.000000%')
#
#         # conversion to string should fail
#         self.assertRaises(ValueError, format, 3.0, "s")
#
#         # other format specifiers shouldn't work on floats,
#         #  in particular int specifiers
#         for format_spec in ([chr(x) for x in range(ord('a'), ord('z')+1)] +
#                             [chr(x) for x in range(ord('A'), ord('Z')+1)]):
#             if not format_spec in 'eEfFgGn%':
#                 self.assertRaises(ValueError, format, 0.0, format_spec)
#                 self.assertRaises(ValueError, format, 1.0, format_spec)
#                 self.assertRaises(ValueError, format, -1.0, format_spec)
#                 self.assertRaises(ValueError, format, 1e100, format_spec)
#                 self.assertRaises(ValueError, format, -1e100, format_spec)
#                 self.assertRaises(ValueError, format, 1e-100, format_spec)
#                 self.assertRaises(ValueError, format, -1e-100, format_spec)
#
#         # issue 3382
#         self.assertEqual(format(NAN, 'f'), 'nan')
#         self.assertEqual(format(NAN, 'F'), 'NAN')
#         self.assertEqual(format(INF, 'f'), 'inf')
#         self.assertEqual(format(INF, 'F'), 'INF')
#
#     @support.requires_IEEE_754
#     def test_format_testfile(self):
#         with open(format_testfile) as testfile:
#             for line in testfile:
#                 if line.startswith('--'):
#                     continue
#                 line = line.strip()
#                 if not line:
#                     continue
#
#                 lhs, rhs = map(str.strip, line.split('->'))
#                 fmt, arg = lhs.split()
#                 self.assertEqual(fmt % float(arg), rhs)
#                 self.assertEqual(fmt % -float(arg), '-' + rhs)
#
#     def test_issue5864(self):
#         self.assertEqual(format(123.456, '.4'), '123.5')
#         self.assertEqual(format(1234.56, '.4'), '1.235e+03')
#         self.assertEqual(format(12345.6, '.4'), '1.235e+04')
#
# class ReprTestCase(unittest.TestCase):
#     def test_repr(self):
#         floats_file = open(os.path.join(os.path.split(__file__)[0],
#                            'floating_points.txt'))
#         for line in floats_file:
#             line = line.strip()
#             if not line or line.startswith('#'):
#                 continue
#             v = eval(line)
#             self.assertEqual(v, eval(repr(v)))
#         floats_file.close()
#
#     @unittest.skipUnless(getattr(sys, 'float_repr_style', '') == 'short',
#                          "applies only when using short float repr style")
#     def test_short_repr(self):
#         # test short float repr introduced in Python 3.1.  One aspect
#         # of this repr is that we get some degree of str -> float ->
#         # str roundtripping.  In particular, for any numeric string
#         # containing 15 or fewer significant digits, those exact same
#         # digits (modulo trailing zeros) should appear in the output.
#         # No more repr(0.03) -> "0.029999999999999999"!
#
#         test_strings = [
#             # output always includes *either* a decimal point and at
#             # least one digit after that point, or an exponent.
#             '0.0',
#             '1.0',
#             '0.01',
#             '0.02',
#             '0.03',
#             '0.04',
#             '0.05',
#             '1.23456789',
#             '10.0',
#             '100.0',
#             # values >= 1e16 get an exponent...
#             '1000000000000000.0',
#             '9999999999999990.0',
#             '1e+16',
#             '1e+17',
#             # ... and so do values < 1e-4
#             '0.001',
#             '0.001001',
#             '0.00010000000000001',
#             '0.0001',
#             '9.999999999999e-05',
#             '1e-05',
#             # values designed to provoke failure if the FPU rounding
#             # precision isn't set correctly
#             '8.72293771110361e+25',
#             '7.47005307342313e+26',
#             '2.86438000439698e+28',
#             '8.89142905246179e+28',
#             '3.08578087079232e+35',
#             ]
#
#         for s in test_strings:
#             negs = '-'+s
#             self.assertEqual(s, repr(float(s)))
#             self.assertEqual(negs, repr(float(negs)))
#             # Since Python 3.2, repr and str are identical
#             self.assertEqual(repr(float(s)), str(float(s)))
#             self.assertEqual(repr(float(negs)), str(float(negs)))
#
# @support.requires_IEEE_754
# class RoundTestCase(unittest.TestCase):
#
#     def test_inf_nan(self):
#         self.assertRaises(OverflowError, round, INF)
#         self.assertRaises(OverflowError, round, -INF)
#         self.assertRaises(ValueError, round, NAN)
#         self.assertRaises(TypeError, round, INF, 0.0)
#         self.assertRaises(TypeError, round, -INF, 1.0)
#         self.assertRaises(TypeError, round, NAN, "ceci n'est pas un integer")
#         self.assertRaises(TypeError, round, -0.0, 1j)
#
#     def test_large_n(self):
#         for n in [324, 325, 400, 2**31-1, 2**31, 2**32, 2**100]:
#             self.assertEqual(round(123.456, n), 123.456)
#             self.assertEqual(round(-123.456, n), -123.456)
#             self.assertEqual(round(1e300, n), 1e300)
#             self.assertEqual(round(1e-320, n), 1e-320)
#         self.assertEqual(round(1e150, 300), 1e150)
#         self.assertEqual(round(1e300, 307), 1e300)
#         self.assertEqual(round(-3.1415, 308), -3.1415)
#         self.assertEqual(round(1e150, 309), 1e150)
#         self.assertEqual(round(1.4e-315, 315), 1e-315)
#
#     def test_small_n(self):
#         for n in [-308, -309, -400, 1-2**31, -2**31, -2**31-1, -2**100]:
#             self.assertEqual(round(123.456, n), 0.0)
#             self.assertEqual(round(-123.456, n), -0.0)
#             self.assertEqual(round(1e300, n), 0.0)
#             self.assertEqual(round(1e-320, n), 0.0)
#
#     def test_overflow(self):
#         self.assertRaises(OverflowError, round, 1.6e308, -308)
#         self.assertRaises(OverflowError, round, -1.7e308, -308)
#
#     @unittest.skipUnless(getattr(sys, 'float_repr_style', '') == 'short',
#                          "applies only when using short float repr style")
#     def test_previous_round_bugs(self):
#         # particular cases that have occurred in bug reports
#         self.assertEqual(round(562949953421312.5, 1),
#                           562949953421312.5)
#         self.assertEqual(round(56294995342131.5, 3),
#                          56294995342131.5)
#         # round-half-even
#         self.assertEqual(round(25.0, -1), 20.0)
#         self.assertEqual(round(35.0, -1), 40.0)
#         self.assertEqual(round(45.0, -1), 40.0)
#         self.assertEqual(round(55.0, -1), 60.0)
#         self.assertEqual(round(65.0, -1), 60.0)
#         self.assertEqual(round(75.0, -1), 80.0)
#         self.assertEqual(round(85.0, -1), 80.0)
#         self.assertEqual(round(95.0, -1), 100.0)
#
#     @unittest.skipUnless(getattr(sys, 'float_repr_style', '') == 'short',
#                          "applies only when using short float repr style")
#     def test_matches_float_format(self):
#         # round should give the same results as float formatting
#         for i in range(500):
#             x = i/1000.
#             self.assertEqual(float(format(x, '.0f')), round(x, 0))
#             self.assertEqual(float(format(x, '.1f')), round(x, 1))
#             self.assertEqual(float(format(x, '.2f')), round(x, 2))
#             self.assertEqual(float(format(x, '.3f')), round(x, 3))
#
#         for i in range(5, 5000, 10):
#             x = i/1000.
#             self.assertEqual(float(format(x, '.0f')), round(x, 0))
#             self.assertEqual(float(format(x, '.1f')), round(x, 1))
#             self.assertEqual(float(format(x, '.2f')), round(x, 2))
#             self.assertEqual(float(format(x, '.3f')), round(x, 3))
#
#         for i in range(500):
#             x = random.random()
#             self.assertEqual(float(format(x, '.0f')), round(x, 0))
#             self.assertEqual(float(format(x, '.1f')), round(x, 1))
#             self.assertEqual(float(format(x, '.2f')), round(x, 2))
#             self.assertEqual(float(format(x, '.3f')), round(x, 3))
#
#     def test_format_specials(self):
#         # Test formatting of nans and infs.
#
#         def test(fmt, value, expected):
#             # Test with both % and format().
#             self.assertEqual(fmt % value, expected, fmt)
#             fmt = fmt[1:] # strip off the %
#             self.assertEqual(format(value, fmt), expected, fmt)
#
#         for fmt in ['%e', '%f', '%g', '%.0e', '%.6f', '%.20g',
#                     '%#e', '%#f', '%#g', '%#.20e', '%#.15f', '%#.3g']:
#             pfmt = '%+' + fmt[1:]
#             sfmt = '% ' + fmt[1:]
#             test(fmt, INF, 'inf')
#             test(fmt, -INF, '-inf')
#             test(fmt, NAN, 'nan')
#             test(fmt, -NAN, 'nan')
#             # When asking for a sign, it's always provided. nans are
#             #  always positive.
#             test(pfmt, INF, '+inf')
#             test(pfmt, -INF, '-inf')
#             test(pfmt, NAN, '+nan')
#             test(pfmt, -NAN, '+nan')
#             # When using ' ' for a sign code, only infs can be negative.
#             #  Others have a space.
#             test(sfmt, INF, ' inf')
#             test(sfmt, -INF, '-inf')
#             test(sfmt, NAN, ' nan')
#             test(sfmt, -NAN, ' nan')


# Beginning with Python 2.6 float has cross platform compatible
# ways to create and represent inf and nan
class InfNanTest(unittest.TestCase):
    def test_inf_from_str(self):
        self.assertTrue(isinf(float("inf")))
        self.assertTrue(isinf(float("+inf")))
        self.assertTrue(isinf(float("-inf")))
        self.assertTrue(isinf(float("infinity")))
        self.assertTrue(isinf(float("+infinity")))
        self.assertTrue(isinf(float("-infinity")))

        self.assertEqual(repr(float("inf")), "inf")
        self.assertEqual(repr(float("+inf")), "inf")
        self.assertEqual(repr(float("-inf")), "-inf")
        self.assertEqual(repr(float("infinity")), "inf")
        self.assertEqual(repr(float("+infinity")), "inf")
        self.assertEqual(repr(float("-infinity")), "-inf")

        self.assertEqual(repr(float("INF")), "inf")
        self.assertEqual(repr(float("+Inf")), "inf")
        self.assertEqual(repr(float("-iNF")), "-inf")
        self.assertEqual(repr(float("Infinity")), "inf")
        self.assertEqual(repr(float("+iNfInItY")), "inf")
        self.assertEqual(repr(float("-INFINITY")), "-inf")

        self.assertEqual(str(float("inf")), "inf")
        self.assertEqual(str(float("+inf")), "inf")
        self.assertEqual(str(float("-inf")), "-inf")
        self.assertEqual(str(float("infinity")), "inf")
        self.assertEqual(str(float("+infinity")), "inf")
        self.assertEqual(str(float("-infinity")), "-inf")

        self.assertRaises(ValueError, float, "info")
        self.assertRaises(ValueError, float, "+info")
        self.assertRaises(ValueError, float, "-info")
        self.assertRaises(ValueError, float, "in")
        self.assertRaises(ValueError, float, "+in")
        self.assertRaises(ValueError, float, "-in")
        self.assertRaises(ValueError, float, "infinit")
        self.assertRaises(ValueError, float, "+Infin")
        self.assertRaises(ValueError, float, "-INFI")
        self.assertRaises(ValueError, float, "infinitys")

        self.assertRaises(ValueError, float, "++Inf")
        self.assertRaises(ValueError, float, "-+inf")
        self.assertRaises(ValueError, float, "+-infinity")
        self.assertRaises(ValueError, float, "--Infinity")

    # def test_inf_as_str(self):
    #     self.assertEqual(repr(1e300 * 1e300), "inf")
    #     self.assertEqual(repr(-1e300 * 1e300), "-inf")
    #
    #     self.assertEqual(str(1e300 * 1e300), "inf")
    #     self.assertEqual(str(-1e300 * 1e300), "-inf")

    def test_nan_from_str(self):
        self.assertTrue(isnan(float("nan")))
        self.assertTrue(isnan(float("+nan")))
        self.assertTrue(isnan(float("-nan")))

        self.assertEqual(repr(float("nan")), "nan")
        self.assertEqual(repr(float("+nan")), "nan")
        self.assertEqual(repr(float("-nan")), "nan")

        self.assertEqual(repr(float("NAN")), "nan")
        self.assertEqual(repr(float("+NAn")), "nan")
        self.assertEqual(repr(float("-NaN")), "nan")

        self.assertEqual(str(float("nan")), "nan")
        self.assertEqual(str(float("+nan")), "nan")
        self.assertEqual(str(float("-nan")), "nan")

        self.assertRaises(ValueError, float, "nana")
        self.assertRaises(ValueError, float, "+nana")
        self.assertRaises(ValueError, float, "-nana")
        self.assertRaises(ValueError, float, "na")
        self.assertRaises(ValueError, float, "+na")
        self.assertRaises(ValueError, float, "-na")

        self.assertRaises(ValueError, float, "++nan")
        self.assertRaises(ValueError, float, "-+NAN")
        self.assertRaises(ValueError, float, "+-NaN")
        self.assertRaises(ValueError, float, "--nAn")

    # def test_nan_as_str(self):
    #     self.assertEqual(repr(1e300 * 1e300 * 0), "nan")
    #     self.assertEqual(repr(-1e300 * 1e300 * 0), "nan")
    #
    #     self.assertEqual(str(1e300 * 1e300 * 0), "nan")
    #     self.assertEqual(str(-1e300 * 1e300 * 0), "nan")

    def test_inf_signs(self):
        self.assertEqual(copysign(1.0, float('inf')), 1.0)
        self.assertEqual(copysign(1.0, float('-inf')), -1.0)

    @unittest.skipUnless(getattr(sys, 'float_repr_style', '') == 'short',
                         "applies only when using short float repr style")
    def test_nan_signs(self):
        # When using the dtoa.c code, the sign of float('nan') should
        # be predictable.
        self.assertEqual(copysign(1.0, float('nan')), 1.0)
        self.assertEqual(copysign(1.0, float('-nan')), -1.0)


# fromHex = float.fromhex
# toHex = float.hex
# class HexFloatTestCase(unittest.TestCase):
#     MAX = fromHex('0x.fffffffffffff8p+1024')  # max normal
#     MIN = fromHex('0x1p-1022')                # min normal
#     TINY = fromHex('0x0.0000000000001p-1022') # min subnormal
#     EPS = fromHex('0x0.0000000000001p0') # diff between 1.0 and next float up
#
#     def identical(self, x, y):
#         # check that floats x and y are identical, or that both
#         # are NaNs
#         if isnan(x) or isnan(y):
#             if isnan(x) == isnan(y):
#                 return
#         elif x == y and (x != 0.0 or copysign(1.0, x) == copysign(1.0, y)):
#             return
#         self.fail('%r not identical to %r' % (x, y))
#
#     def test_ends(self):
#         self.identical(self.MIN, ldexp(1.0, -1022))
#         self.identical(self.TINY, ldexp(1.0, -1074))
#         self.identical(self.EPS, ldexp(1.0, -52))
#         self.identical(self.MAX, 2.*(ldexp(1.0, 1023) - ldexp(1.0, 970)))
#
#     def test_invalid_inputs(self):
#         invalid_inputs = [
#             'infi',   # misspelt infinities and nans
#             '-Infinit',
#             '++inf',
#             '-+Inf',
#             '--nan',
#             '+-NaN',
#             'snan',
#             'NaNs',
#             'nna',
#             'an',
#             'nf',
#             'nfinity',
#             'inity',
#             'iinity',
#             '0xnan',
#             '',
#             ' ',
#             'x1.0p0',
#             '0xX1.0p0',
#             '+ 0x1.0p0', # internal whitespace
#             '- 0x1.0p0',
#             '0 x1.0p0',
#             '0x 1.0p0',
#             '0x1 2.0p0',
#             '+0x1 .0p0',
#             '0x1. 0p0',
#             '-0x1.0 1p0',
#             '-0x1.0 p0',
#             '+0x1.0p +0',
#             '0x1.0p -0',
#             '0x1.0p 0',
#             '+0x1.0p+ 0',
#             '-0x1.0p- 0',
#             '++0x1.0p-0', # double signs
#             '--0x1.0p0',
#             '+-0x1.0p+0',
#             '-+0x1.0p0',
#             '0x1.0p++0',
#             '+0x1.0p+-0',
#             '-0x1.0p-+0',
#             '0x1.0p--0',
#             '0x1.0.p0',
#             '0x.p0', # no hex digits before or after point
#             '0x1,p0', # wrong decimal point character
#             '0x1pa',
#             '0x1p\uff10',  # fullwidth Unicode digits
#             '\uff10x1p0',
#             '0x\uff11p0',
#             '0x1.\uff10p0',
#             '0x1p0 \n 0x2p0',
#             '0x1p0\0 0x1p0',  # embedded null byte is not end of string
#             ]
#         for x in invalid_inputs:
#             try:
#                 result = fromHex(x)
#             except ValueError:
#                 pass
#             else:
#                 self.fail('Expected float.fromhex(%r) to raise ValueError; '
#                           'got %r instead' % (x, result))
#
#
#     def test_whitespace(self):
#         value_pairs = [
#             ('inf', INF),
#             ('-Infinity', -INF),
#             ('nan', NAN),
#             ('1.0', 1.0),
#             ('-0x.2', -0.125),
#             ('-0.0', -0.0)
#             ]
#         whitespace = [
#             '',
#             ' ',
#             '\t',
#             '\n',
#             '\n \t',
#             '\f',
#             '\v',
#             '\r'
#             ]
#         for inp, expected in value_pairs:
#             for lead in whitespace:
#                 for trail in whitespace:
#                     got = fromHex(lead + inp + trail)
#                     self.identical(got, expected)
#
#
#     def test_from_hex(self):
#         MIN = self.MIN;
#         MAX = self.MAX;
#         TINY = self.TINY;
#         EPS = self.EPS;
#
#         # two spellings of infinity, with optional signs; case-insensitive
#         self.identical(fromHex('inf'), INF)
#         self.identical(fromHex('+Inf'), INF)
#         self.identical(fromHex('-INF'), -INF)
#         self.identical(fromHex('iNf'), INF)
#         self.identical(fromHex('Infinity'), INF)
#         self.identical(fromHex('+INFINITY'), INF)
#         self.identical(fromHex('-infinity'), -INF)
#         self.identical(fromHex('-iNFiNitY'), -INF)
#
#         # nans with optional sign; case insensitive
#         self.identical(fromHex('nan'), NAN)
#         self.identical(fromHex('+NaN'), NAN)
#         self.identical(fromHex('-NaN'), NAN)
#         self.identical(fromHex('-nAN'), NAN)
#
#         # variations in input format
#         self.identical(fromHex('1'), 1.0)
#         self.identical(fromHex('+1'), 1.0)
#         self.identical(fromHex('1.'), 1.0)
#         self.identical(fromHex('1.0'), 1.0)
#         self.identical(fromHex('1.0p0'), 1.0)
#         self.identical(fromHex('01'), 1.0)
#         self.identical(fromHex('01.'), 1.0)
#         self.identical(fromHex('0x1'), 1.0)
#         self.identical(fromHex('0x1.'), 1.0)
#         self.identical(fromHex('0x1.0'), 1.0)
#         self.identical(fromHex('+0x1.0'), 1.0)
#         self.identical(fromHex('0x1p0'), 1.0)
#         self.identical(fromHex('0X1p0'), 1.0)
#         self.identical(fromHex('0X1P0'), 1.0)
#         self.identical(fromHex('0x1P0'), 1.0)
#         self.identical(fromHex('0x1.p0'), 1.0)
#         self.identical(fromHex('0x1.0p0'), 1.0)
#         self.identical(fromHex('0x.1p4'), 1.0)
#         self.identical(fromHex('0x.1p04'), 1.0)
#         self.identical(fromHex('0x.1p004'), 1.0)
#         self.identical(fromHex('0x1p+0'), 1.0)
#         self.identical(fromHex('0x1P-0'), 1.0)
#         self.identical(fromHex('+0x1p0'), 1.0)
#         self.identical(fromHex('0x01p0'), 1.0)
#         self.identical(fromHex('0x1p00'), 1.0)
#         self.identical(fromHex(' 0x1p0 '), 1.0)
#         self.identical(fromHex('\n 0x1p0'), 1.0)
#         self.identical(fromHex('0x1p0 \t'), 1.0)
#         self.identical(fromHex('0xap0'), 10.0)
#         self.identical(fromHex('0xAp0'), 10.0)
#         self.identical(fromHex('0xaP0'), 10.0)
#         self.identical(fromHex('0xAP0'), 10.0)
#         self.identical(fromHex('0xbep0'), 190.0)
#         self.identical(fromHex('0xBep0'), 190.0)
#         self.identical(fromHex('0xbEp0'), 190.0)
#         self.identical(fromHex('0XBE0P-4'), 190.0)
#         self.identical(fromHex('0xBEp0'), 190.0)
#         self.identical(fromHex('0xB.Ep4'), 190.0)
#         self.identical(fromHex('0x.BEp8'), 190.0)
#         self.identical(fromHex('0x.0BEp12'), 190.0)
#
#         # moving the point around
#         pi = fromHex('0x1.921fb54442d18p1')
#         self.identical(fromHex('0x.006487ed5110b46p11'), pi)
#         self.identical(fromHex('0x.00c90fdaa22168cp10'), pi)
#         self.identical(fromHex('0x.01921fb54442d18p9'), pi)
#         self.identical(fromHex('0x.03243f6a8885a3p8'), pi)
#         self.identical(fromHex('0x.06487ed5110b46p7'), pi)
#         self.identical(fromHex('0x.0c90fdaa22168cp6'), pi)
#         self.identical(fromHex('0x.1921fb54442d18p5'), pi)
#         self.identical(fromHex('0x.3243f6a8885a3p4'), pi)
#         self.identical(fromHex('0x.6487ed5110b46p3'), pi)
#         self.identical(fromHex('0x.c90fdaa22168cp2'), pi)
#         self.identical(fromHex('0x1.921fb54442d18p1'), pi)
#         self.identical(fromHex('0x3.243f6a8885a3p0'), pi)
#         self.identical(fromHex('0x6.487ed5110b46p-1'), pi)
#         self.identical(fromHex('0xc.90fdaa22168cp-2'), pi)
#         self.identical(fromHex('0x19.21fb54442d18p-3'), pi)
#         self.identical(fromHex('0x32.43f6a8885a3p-4'), pi)
#         self.identical(fromHex('0x64.87ed5110b46p-5'), pi)
#         self.identical(fromHex('0xc9.0fdaa22168cp-6'), pi)
#         self.identical(fromHex('0x192.1fb54442d18p-7'), pi)
#         self.identical(fromHex('0x324.3f6a8885a3p-8'), pi)
#         self.identical(fromHex('0x648.7ed5110b46p-9'), pi)
#         self.identical(fromHex('0xc90.fdaa22168cp-10'), pi)
#         self.identical(fromHex('0x1921.fb54442d18p-11'), pi)
#         # ...
#         self.identical(fromHex('0x1921fb54442d1.8p-47'), pi)
#         self.identical(fromHex('0x3243f6a8885a3p-48'), pi)
#         self.identical(fromHex('0x6487ed5110b46p-49'), pi)
#         self.identical(fromHex('0xc90fdaa22168cp-50'), pi)
#         self.identical(fromHex('0x1921fb54442d18p-51'), pi)
#         self.identical(fromHex('0x3243f6a8885a30p-52'), pi)
#         self.identical(fromHex('0x6487ed5110b460p-53'), pi)
#         self.identical(fromHex('0xc90fdaa22168c0p-54'), pi)
#         self.identical(fromHex('0x1921fb54442d180p-55'), pi)
#
#
#         # results that should overflow...
#         self.assertRaises(OverflowError, fromHex, '-0x1p1024')
#         self.assertRaises(OverflowError, fromHex, '0x1p+1025')
#         self.assertRaises(OverflowError, fromHex, '+0X1p1030')
#         self.assertRaises(OverflowError, fromHex, '-0x1p+1100')
#         self.assertRaises(OverflowError, fromHex, '0X1p123456789123456789')
#         self.assertRaises(OverflowError, fromHex, '+0X.8p+1025')
#         self.assertRaises(OverflowError, fromHex, '+0x0.8p1025')
#         self.assertRaises(OverflowError, fromHex, '-0x0.4p1026')
#         self.assertRaises(OverflowError, fromHex, '0X2p+1023')
#         self.assertRaises(OverflowError, fromHex, '0x2.p1023')
#         self.assertRaises(OverflowError, fromHex, '-0x2.0p+1023')
#         self.assertRaises(OverflowError, fromHex, '+0X4p+1022')
#         self.assertRaises(OverflowError, fromHex, '0x1.ffffffffffffffp+1023')
#         self.assertRaises(OverflowError, fromHex, '-0X1.fffffffffffff9p1023')
#         self.assertRaises(OverflowError, fromHex, '0X1.fffffffffffff8p1023')
#         self.assertRaises(OverflowError, fromHex, '+0x3.fffffffffffffp1022')
#         self.assertRaises(OverflowError, fromHex, '0x3fffffffffffffp+970')
#         self.assertRaises(OverflowError, fromHex, '0x10000000000000000p960')
#         self.assertRaises(OverflowError, fromHex, '-0Xffffffffffffffffp960')
#
#         # ...and those that round to +-max float
#         self.identical(fromHex('+0x1.fffffffffffffp+1023'), MAX)
#         self.identical(fromHex('-0X1.fffffffffffff7p1023'), -MAX)
#         self.identical(fromHex('0X1.fffffffffffff7fffffffffffffp1023'), MAX)
#
#         # zeros
#         self.identical(fromHex('0x0p0'), 0.0)
#         self.identical(fromHex('0x0p1000'), 0.0)
#         self.identical(fromHex('-0x0p1023'), -0.0)
#         self.identical(fromHex('0X0p1024'), 0.0)
#         self.identical(fromHex('-0x0p1025'), -0.0)
#         self.identical(fromHex('0X0p2000'), 0.0)
#         self.identical(fromHex('0x0p123456789123456789'), 0.0)
#         self.identical(fromHex('-0X0p-0'), -0.0)
#         self.identical(fromHex('-0X0p-1000'), -0.0)
#         self.identical(fromHex('0x0p-1023'), 0.0)
#         self.identical(fromHex('-0X0p-1024'), -0.0)
#         self.identical(fromHex('-0x0p-1025'), -0.0)
#         self.identical(fromHex('-0x0p-1072'), -0.0)
#         self.identical(fromHex('0X0p-1073'), 0.0)
#         self.identical(fromHex('-0x0p-1074'), -0.0)
#         self.identical(fromHex('0x0p-1075'), 0.0)
#         self.identical(fromHex('0X0p-1076'), 0.0)
#         self.identical(fromHex('-0X0p-2000'), -0.0)
#         self.identical(fromHex('-0x0p-123456789123456789'), -0.0)
#
#         # values that should underflow to 0
#         self.identical(fromHex('0X1p-1075'), 0.0)
#         self.identical(fromHex('-0X1p-1075'), -0.0)
#         self.identical(fromHex('-0x1p-123456789123456789'), -0.0)
#         self.identical(fromHex('0x1.00000000000000001p-1075'), TINY)
#         self.identical(fromHex('-0x1.1p-1075'), -TINY)
#         self.identical(fromHex('0x1.fffffffffffffffffp-1075'), TINY)
#
#         # check round-half-even is working correctly near 0 ...
#         self.identical(fromHex('0x1p-1076'), 0.0)
#         self.identical(fromHex('0X2p-1076'), 0.0)
#         self.identical(fromHex('0X3p-1076'), TINY)
#         self.identical(fromHex('0x4p-1076'), TINY)
#         self.identical(fromHex('0X5p-1076'), TINY)
#         self.identical(fromHex('0X6p-1076'), 2*TINY)
#         self.identical(fromHex('0x7p-1076'), 2*TINY)
#         self.identical(fromHex('0X8p-1076'), 2*TINY)
#         self.identical(fromHex('0X9p-1076'), 2*TINY)
#         self.identical(fromHex('0xap-1076'), 2*TINY)
#         self.identical(fromHex('0Xbp-1076'), 3*TINY)
#         self.identical(fromHex('0xcp-1076'), 3*TINY)
#         self.identical(fromHex('0Xdp-1076'), 3*TINY)
#         self.identical(fromHex('0Xep-1076'), 4*TINY)
#         self.identical(fromHex('0xfp-1076'), 4*TINY)
#         self.identical(fromHex('0x10p-1076'), 4*TINY)
#         self.identical(fromHex('-0x1p-1076'), -0.0)
#         self.identical(fromHex('-0X2p-1076'), -0.0)
#         self.identical(fromHex('-0x3p-1076'), -TINY)
#         self.identical(fromHex('-0X4p-1076'), -TINY)
#         self.identical(fromHex('-0x5p-1076'), -TINY)
#         self.identical(fromHex('-0x6p-1076'), -2*TINY)
#         self.identical(fromHex('-0X7p-1076'), -2*TINY)
#         self.identical(fromHex('-0X8p-1076'), -2*TINY)
#         self.identical(fromHex('-0X9p-1076'), -2*TINY)
#         self.identical(fromHex('-0Xap-1076'), -2*TINY)
#         self.identical(fromHex('-0xbp-1076'), -3*TINY)
#         self.identical(fromHex('-0xcp-1076'), -3*TINY)
#         self.identical(fromHex('-0Xdp-1076'), -3*TINY)
#         self.identical(fromHex('-0xep-1076'), -4*TINY)
#         self.identical(fromHex('-0Xfp-1076'), -4*TINY)
#         self.identical(fromHex('-0X10p-1076'), -4*TINY)
#
#         # ... and near MIN ...
#         self.identical(fromHex('0x0.ffffffffffffd6p-1022'), MIN-3*TINY)
#         self.identical(fromHex('0x0.ffffffffffffd8p-1022'), MIN-2*TINY)
#         self.identical(fromHex('0x0.ffffffffffffdap-1022'), MIN-2*TINY)
#         self.identical(fromHex('0x0.ffffffffffffdcp-1022'), MIN-2*TINY)
#         self.identical(fromHex('0x0.ffffffffffffdep-1022'), MIN-2*TINY)
#         self.identical(fromHex('0x0.ffffffffffffe0p-1022'), MIN-2*TINY)
#         self.identical(fromHex('0x0.ffffffffffffe2p-1022'), MIN-2*TINY)
#         self.identical(fromHex('0x0.ffffffffffffe4p-1022'), MIN-2*TINY)
#         self.identical(fromHex('0x0.ffffffffffffe6p-1022'), MIN-2*TINY)
#         self.identical(fromHex('0x0.ffffffffffffe8p-1022'), MIN-2*TINY)
#         self.identical(fromHex('0x0.ffffffffffffeap-1022'), MIN-TINY)
#         self.identical(fromHex('0x0.ffffffffffffecp-1022'), MIN-TINY)
#         self.identical(fromHex('0x0.ffffffffffffeep-1022'), MIN-TINY)
#         self.identical(fromHex('0x0.fffffffffffff0p-1022'), MIN-TINY)
#         self.identical(fromHex('0x0.fffffffffffff2p-1022'), MIN-TINY)
#         self.identical(fromHex('0x0.fffffffffffff4p-1022'), MIN-TINY)
#         self.identical(fromHex('0x0.fffffffffffff6p-1022'), MIN-TINY)
#         self.identical(fromHex('0x0.fffffffffffff8p-1022'), MIN)
#         self.identical(fromHex('0x0.fffffffffffffap-1022'), MIN)
#         self.identical(fromHex('0x0.fffffffffffffcp-1022'), MIN)
#         self.identical(fromHex('0x0.fffffffffffffep-1022'), MIN)
#         self.identical(fromHex('0x1.00000000000000p-1022'), MIN)
#         self.identical(fromHex('0x1.00000000000002p-1022'), MIN)
#         self.identical(fromHex('0x1.00000000000004p-1022'), MIN)
#         self.identical(fromHex('0x1.00000000000006p-1022'), MIN)
#         self.identical(fromHex('0x1.00000000000008p-1022'), MIN)
#         self.identical(fromHex('0x1.0000000000000ap-1022'), MIN+TINY)
#         self.identical(fromHex('0x1.0000000000000cp-1022'), MIN+TINY)
#         self.identical(fromHex('0x1.0000000000000ep-1022'), MIN+TINY)
#         self.identical(fromHex('0x1.00000000000010p-1022'), MIN+TINY)
#         self.identical(fromHex('0x1.00000000000012p-1022'), MIN+TINY)
#         self.identical(fromHex('0x1.00000000000014p-1022'), MIN+TINY)
#         self.identical(fromHex('0x1.00000000000016p-1022'), MIN+TINY)
#         self.identical(fromHex('0x1.00000000000018p-1022'), MIN+2*TINY)
#
#         # ... and near 1.0.
#         self.identical(fromHex('0x0.fffffffffffff0p0'), 1.0-EPS)
#         self.identical(fromHex('0x0.fffffffffffff1p0'), 1.0-EPS)
#         self.identical(fromHex('0X0.fffffffffffff2p0'), 1.0-EPS)
#         self.identical(fromHex('0x0.fffffffffffff3p0'), 1.0-EPS)
#         self.identical(fromHex('0X0.fffffffffffff4p0'), 1.0-EPS)
#         self.identical(fromHex('0X0.fffffffffffff5p0'), 1.0-EPS/2)
#         self.identical(fromHex('0X0.fffffffffffff6p0'), 1.0-EPS/2)
#         self.identical(fromHex('0x0.fffffffffffff7p0'), 1.0-EPS/2)
#         self.identical(fromHex('0x0.fffffffffffff8p0'), 1.0-EPS/2)
#         self.identical(fromHex('0X0.fffffffffffff9p0'), 1.0-EPS/2)
#         self.identical(fromHex('0X0.fffffffffffffap0'), 1.0-EPS/2)
#         self.identical(fromHex('0x0.fffffffffffffbp0'), 1.0-EPS/2)
#         self.identical(fromHex('0X0.fffffffffffffcp0'), 1.0)
#         self.identical(fromHex('0x0.fffffffffffffdp0'), 1.0)
#         self.identical(fromHex('0X0.fffffffffffffep0'), 1.0)
#         self.identical(fromHex('0x0.ffffffffffffffp0'), 1.0)
#         self.identical(fromHex('0X1.00000000000000p0'), 1.0)
#         self.identical(fromHex('0X1.00000000000001p0'), 1.0)
#         self.identical(fromHex('0x1.00000000000002p0'), 1.0)
#         self.identical(fromHex('0X1.00000000000003p0'), 1.0)
#         self.identical(fromHex('0x1.00000000000004p0'), 1.0)
#         self.identical(fromHex('0X1.00000000000005p0'), 1.0)
#         self.identical(fromHex('0X1.00000000000006p0'), 1.0)
#         self.identical(fromHex('0X1.00000000000007p0'), 1.0)
#         self.identical(fromHex('0x1.00000000000007ffffffffffffffffffffp0'),
#                        1.0)
#         self.identical(fromHex('0x1.00000000000008p0'), 1.0)
#         self.identical(fromHex('0x1.00000000000008000000000000000001p0'),
#                        1+EPS)
#         self.identical(fromHex('0X1.00000000000009p0'), 1.0+EPS)
#         self.identical(fromHex('0x1.0000000000000ap0'), 1.0+EPS)
#         self.identical(fromHex('0x1.0000000000000bp0'), 1.0+EPS)
#         self.identical(fromHex('0X1.0000000000000cp0'), 1.0+EPS)
#         self.identical(fromHex('0x1.0000000000000dp0'), 1.0+EPS)
#         self.identical(fromHex('0x1.0000000000000ep0'), 1.0+EPS)
#         self.identical(fromHex('0X1.0000000000000fp0'), 1.0+EPS)
#         self.identical(fromHex('0x1.00000000000010p0'), 1.0+EPS)
#         self.identical(fromHex('0X1.00000000000011p0'), 1.0+EPS)
#         self.identical(fromHex('0x1.00000000000012p0'), 1.0+EPS)
#         self.identical(fromHex('0X1.00000000000013p0'), 1.0+EPS)
#         self.identical(fromHex('0X1.00000000000014p0'), 1.0+EPS)
#         self.identical(fromHex('0x1.00000000000015p0'), 1.0+EPS)
#         self.identical(fromHex('0x1.00000000000016p0'), 1.0+EPS)
#         self.identical(fromHex('0X1.00000000000017p0'), 1.0+EPS)
#         self.identical(fromHex('0x1.00000000000017ffffffffffffffffffffp0'),
#                        1.0+EPS)
#         self.identical(fromHex('0x1.00000000000018p0'), 1.0+2*EPS)
#         self.identical(fromHex('0X1.00000000000018000000000000000001p0'),
#                        1.0+2*EPS)
#         self.identical(fromHex('0x1.00000000000019p0'), 1.0+2*EPS)
#         self.identical(fromHex('0X1.0000000000001ap0'), 1.0+2*EPS)
#         self.identical(fromHex('0X1.0000000000001bp0'), 1.0+2*EPS)
#         self.identical(fromHex('0x1.0000000000001cp0'), 1.0+2*EPS)
#         self.identical(fromHex('0x1.0000000000001dp0'), 1.0+2*EPS)
#         self.identical(fromHex('0x1.0000000000001ep0'), 1.0+2*EPS)
#         self.identical(fromHex('0X1.0000000000001fp0'), 1.0+2*EPS)
#         self.identical(fromHex('0x1.00000000000020p0'), 1.0+2*EPS)
#
#     def test_roundtrip(self):
#         def roundtrip(x):
#             return fromHex(toHex(x))
#
#         for x in [NAN, INF, self.MAX, self.MIN, self.MIN-self.TINY, self.TINY, 0.0]:
#             self.identical(x, roundtrip(x))
#             self.identical(-x, roundtrip(-x))
#
#         # fromHex(toHex(x)) should exactly recover x, for any non-NaN float x.
#         import random
#         for i in range(10000):
#             e = random.randrange(-1200, 1200)
#             m = random.random()
#             s = random.choice([1.0, -1.0])
#             try:
#                 x = s*ldexp(m, e)
#             except OverflowError:
#                 pass
#             else:
#                 self.identical(x, fromHex(toHex(x)))


# def test_main():
#     support.run_unittest(
#         GeneralFloatCases,
#         FormatFunctionsTestCase,
#         UnknownFormatTestCase,
#         IEEEFormatTestCase,
#         FormatTestCase,
#         ReprTestCase,
#         RoundTestCase,
#         InfNanTest,
#         HexFloatTestCase,
#         )

if __name__ == '__main__':
    test_main()



from __future__ import print_function, division

from time_base import comparison_timer

float_re = '''\
import re
float_regex = re.compile(r'[-+]?\d*\.?\d+(?:[eE][-+]?\d+)?$')
float_match = float_regex.match
def float_re(x):
    """Function to simulate safe_float but with regular expressions."""
    try:
        if float_match(x):
            return float(x)
        else:
            return x
    except TypeError:
        return float(x)
'''

float_try = '''\
def float_try(x):
    """Function to simulate safe_float but with try/except."""
    try:
        return float(x)
    except ValueError:
        return x
'''

comparison_timer(['float_re', float_re],
                 ['float_try', float_try],
                 ['fast_float', 'from fastnumbers import fast_float'])
